@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2016 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
-if sys.version_info[:2] < (2, 6) or sys.version_info[:2] >= (3, 0):
-    print "Sorry, requires Python 2.6 or 2.7."
+if sys.version_info[:2] < (2, 7) or sys.version_info[:2] >= (3, 0):
+    print "Sorry, requires Python 2.7."
     sys.exit(1)
 
 # Make sure UTF-8 is default 8bit encoding
@@ -27,7 +27,7 @@ try:
     sys.setdefaultencoding('utf-8')
 except:
     print 'Sorry, you MUST add the SABnzbd folder to the PYTHONPATH environment variable'
-    print 'or find another way to force Python to use UTF-8 for string encoding.'
+    print 'or find another way to force Python to use UTF-8 for text encoding.'
     sys.exit(1)
 
 import logging
@@ -52,12 +52,11 @@ except:
     sys.exit(1)
 
 import cherrypy
-if [int(n) for n in cherrypy.__version__.split('.')] < [3, 8, 0]:
-    print 'Sorry, requires Python module Cherrypy 3.8.0+ (use the included version)'
+if [int(n) for n in cherrypy.__version__.split('.')] < [8, 1, 2]:
+    print 'Sorry, requires Python module Cherrypy 8.1.2+ (use the included version)'
     sys.exit(1)
 
 from cherrypy import _cpserver
-from cherrypy import _cpwsgi_server
 
 SQLITE_DLL = True
 try:
@@ -98,26 +97,14 @@ import sabnzbd.config as config
 import sabnzbd.cfg
 import sabnzbd.downloader
 from sabnzbd.encoding import unicoder, deunicode
-import sabnzbd.growler as growler
+import sabnzbd.notifier as notifier
 import sabnzbd.zconfig
+import sabnzbd.utils.sslinfo
 
 from threading import Thread
 
 LOG_FLAG = False        # Global for this module, signaling loglevel change
 
-_first_log = True
-
-
-def FORCELOG(txt):
-    global _first_log
-    if _first_log:
-        os.remove('d:/temp/debug.txt')
-        _first_log = False
-    ff = open('d:/temp/debug.txt', 'a+')
-    ff.write(txt)
-    ff.write('\n')
-    ff.close()
-
 
 try:
     import win32api
@@ -143,44 +130,6 @@ def guard_loglevel():
     LOG_FLAG = True
 
 
-# Improved RotatingFileHandler
-# See: http://www.mail-archive.com/python-bugs-list@python.org/msg53913.html
-# http://bugs.python.org/file14420/NTSafeLogging.py
-# Thanks Erik Antelman
-#
-if sabnzbd.WIN32:
-
-    import msvcrt
-    import _subprocess
-    import codecs
-
-    def duplicate(handle, inheritable=False):
-        target_process = _subprocess.GetCurrentProcess()
-        return _subprocess.DuplicateHandle(
-            _subprocess.GetCurrentProcess(), handle, target_process,
-            0, inheritable, _subprocess.DUPLICATE_SAME_ACCESS).Detach()
-
-    class NewRotatingFileHandler(logging.handlers.RotatingFileHandler):
-
-        def _open(self):
-            """ Open the current base file with the (original) mode and encoding.
-                Return the resulting stream.
-            """
-            if self.encoding is None:
-                stream = open(self.baseFilename, self.mode)
-                newosf = duplicate(msvcrt.get_osfhandle(stream.fileno()), inheritable=False)
-                newFD = msvcrt.open_osfhandle(newosf, os.O_APPEND)
-                newstream = os.fdopen(newFD, self.mode)
-                stream.close()
-                return newstream
-            else:
-                stream = codecs.open(self.baseFilename, self.mode, self.encoding)
-            return stream
-
-else:
-    NewRotatingFileHandler = logging.handlers.RotatingFileHandler
-
-
 class FilterCP3:
     # Filter out all CherryPy3-Access logging that we receive,
     # because we have the root logger
@@ -285,7 +234,7 @@ def print_version():
     print """
 %s-%s
 
-Copyright (C) 2008-2016, The SABnzbd-Team <team@sabnzbd.org>
+Copyright (C) 2008-2017, The SABnzbd-Team <team@sabnzbd.org>
 SABnzbd comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. It is licensed under the
@@ -482,10 +431,12 @@ def print_modules():
     if sabnzbd.decoder.HAVE_YENC:
         logging.info("_yenc module... found!")
     else:
-        if hasattr(sys, "frozen"):
-            logging.error(T('_yenc module... NOT found!'))
-        else:
-            logging.info("_yenc module... NOT found!")
+        logging.warning(T('_yenc module... NOT found!'))
+
+    if sabnzbd.HAVE_CRYPTOGRAPHY:
+        logging.info('Cryptography module (v%s)... found!', sabnzbd.HAVE_CRYPTOGRAPHY)
+    else:
+        logging.info('Cryptography module... NOT found!')
 
     if sabnzbd.newsunpack.PAR2_COMMAND:
         logging.info("par2 binary... found (%s)", sabnzbd.newsunpack.PAR2_COMMAND)
@@ -493,7 +444,7 @@ def print_modules():
         logging.error(T('par2 binary... NOT found!'))
 
     if sabnzbd.newsunpack.PAR2C_COMMAND:
-        logging.info("par2-classic binary... found (%s)", sabnzbd.newsunpack.PAR2C_COMMAND)
+        logging.info("par2cmdline binary... found (%s)", sabnzbd.newsunpack.PAR2C_COMMAND)
 
     if sabnzbd.newsunpack.RAR_COMMAND:
         logging.info("UNRAR binary... found (%s)", sabnzbd.newsunpack.RAR_COMMAND)
@@ -504,14 +455,12 @@ def print_modules():
     if sabnzbd.newsunpack.ZIP_COMMAND:
         logging.info("unzip binary... found (%s)", sabnzbd.newsunpack.ZIP_COMMAND)
     else:
-        if sabnzbd.cfg.enable_unzip():
-            logging.warning(T('unzip binary... NOT found!'))
+        logging.info(T('unzip binary... NOT found!'))
 
     if sabnzbd.newsunpack.SEVEN_COMMAND:
         logging.info("7za binary... found (%s)", sabnzbd.newsunpack.SEVEN_COMMAND)
     else:
-        if sabnzbd.cfg.enable_7zip():
-            logging.info(T('7za binary... NOT found!'))
+        logging.info(T('7za binary... NOT found!'))
 
     if not sabnzbd.WIN32:
         if sabnzbd.newsunpack.NICE_COMMAND:
@@ -523,11 +472,6 @@ def print_modules():
         else:
             logging.info("ionice binary... NOT found!")
 
-    if sabnzbd.newswrapper.HAVE_SSL:
-        logging.info("pyOpenSSL... found (%s)", sabnzbd.newswrapper.HAVE_SSL)
-    else:
-        logging.info("pyOpenSSL... NOT found! - Try apt-get install python-openssl (SSL is optional)")
-
 
 def all_localhosts():
     """ Return all unique values of localhost in order of preference """
@@ -548,7 +492,11 @@ def all_localhosts():
     ips = []
     for item in info:
         item = item[4][0]
-        if item not in ips:
+        # Avoid problems on strange Linux settings
+        if not isinstance(item, basestring):
+            continue
+        # Only return IPv6 when enabled
+        if item not in ips and ('::1' not in item or sabnzbd.cfg.ipv6_hosting()):
             ips.append(item)
     return ips
 
@@ -691,7 +639,7 @@ def get_webhost(cherryhost, cherryport, https_port):
 
     if cherryport == https_port and sabnzbd.cfg.enable_https():
         sabnzbd.cfg.enable_https.set(False)
-        # TODO: Should have a translated message, but that's not available yet
+        # Should have a translated message, but that's not available yet
         logging.error(T('HTTP and HTTPS ports cannot be the same'))
 
     return cherryhost, cherryport, browserhost, https_port
@@ -700,25 +648,25 @@ def get_webhost(cherryhost, cherryport, https_port):
 def attach_server(host, port, cert=None, key=None, chain=None):
     """ Define and attach server, optionally HTTPS """
     if sabnzbd.cfg.ipv6_hosting() or '::1' not in host:
-        http_server = _cpwsgi_server.CPWSGIServer()
+        http_server = cherrypy._cpserver.Server()
         http_server.bind_addr = (host, port)
         if cert and key:
+            http_server.ssl_provider = 'builtin'
             http_server.ssl_certificate = cert
             http_server.ssl_private_key = key
             http_server.ssl_certificate_chain = chain
-        adapter = _cpserver.ServerAdapter(cherrypy.engine, http_server, http_server.bind_addr)
-        adapter.subscribe()
+        http_server.subscribe()
 
 
-def is_sabnzbd_running(url, timeout=None):
+def is_sabnzbd_running(url):
     """ Return True when there's already a SABnzbd instance running. """
     try:
         url = '%s&mode=version' % (url)
         # Do this without certificate verification, few installations will have that
         prev = sabnzbd.set_https_verification(False)
-        ver = sabnzbd.newsunpack.get_from_url(url, timeout=timeout)
+        ver = sabnzbd.newsunpack.get_from_url(url)
         sabnzbd.set_https_verification(prev)
-        return bool(ver and re.search(r'\d+\.\d+\.', ver))
+        return (ver and (re.search(r'\d+\.\d+\.', ver) or ver.strip() == sabnzbd.__version__))
     except:
         return False
 
@@ -728,7 +676,7 @@ def find_free_port(host, currentport):
     n = 0
     while n < 10 and currentport <= 49151:
         try:
-            cherrypy.process.servers.check_port(host, currentport)
+            cherrypy.process.servers.check_port(host, currentport, timeout=0.025)
             return currentport
         except:
             currentport += 5
@@ -835,7 +783,7 @@ def commandline_handler(frozen=True):
                                     'weblogging=', 'server=', 'templates', 'ipv6_hosting=',
                                     'template2', 'browser=', 'config-file=', 'force',
                                     'version', 'https=', 'autorestarted', 'repair', 'repair-all',
-                                    'log-all', 'no-login', 'pid=', 'new', 'sessions', 'console', 'pidfile=',
+                                    'log-all', 'no-login', 'pid=', 'new', 'console', 'pidfile=',
                                     # Below Win32 Service options
                                     'password=', 'username=', 'startup=', 'perfmonini=', 'perfmondll=',
                                     'interactive', 'wait=',
@@ -904,11 +852,10 @@ def main():
     repair = 0
     api_url = None
     no_login = False
-    re_argv = [sys.argv[0]]
+    sabnzbd.RESTART_ARGS = [sys.argv[0]]
     pid_path = None
     pid_file = None
     new_instance = False
-    force_sessions = False
     osx_console = False
     ipv6_hosting = None
 
@@ -922,11 +869,11 @@ def main():
                 fork = True
             autobrowser = False
             sabnzbd.DAEMON = True
-            re_argv.append(opt)
+            sabnzbd.RESTART_ARGS.append(opt)
         elif opt in ('-f', '--config-file'):
             inifile = arg
-            re_argv.append(opt)
-            re_argv.append(arg)
+            sabnzbd.RESTART_ARGS.append(opt)
+            sabnzbd.RESTART_ARGS.append(arg)
         elif opt in ('-h', '--help'):
             print_help()
             exit_sab(0)
@@ -943,7 +890,7 @@ def main():
                 autobrowser = bool(int(arg))
             except:
                 autobrowser = True
-        elif opt in ('--autorestarted'):
+        elif opt in ('--autorestarted', ):
             autorestarted = True
         elif opt in ('-c', '--clean'):
             clean_up = True
@@ -970,11 +917,11 @@ def main():
             pause = True
         elif opt in ('--force',):
             force_web = True
-            re_argv.append(opt)
+            sabnzbd.RESTART_ARGS.append(opt)
         elif opt in ('--https',):
             https_port = int(arg)
-            re_argv.append(opt)
-            re_argv.append(arg)
+            sabnzbd.RESTART_ARGS.append(opt)
+            sabnzbd.RESTART_ARGS.append(arg)
         elif opt in ('--repair',):
             repair = 1
             pause = True
@@ -987,23 +934,20 @@ def main():
             no_login = True
         elif opt in ('--pid',):
             pid_path = arg
-            re_argv.append(opt)
-            re_argv.append(arg)
+            sabnzbd.RESTART_ARGS.append(opt)
+            sabnzbd.RESTART_ARGS.append(arg)
         elif opt in ('--pidfile',):
             pid_file = arg
-            re_argv.append(opt)
-            re_argv.append(arg)
+            sabnzbd.RESTART_ARGS.append(opt)
+            sabnzbd.RESTART_ARGS.append(arg)
         elif opt in ('--new',):
             new_instance = True
-        elif opt in ('--sessions',):
-            re_argv.append(opt)
-            force_sessions = True
         elif opt in ('--console',):
-            re_argv.append(opt)
+            sabnzbd.RESTART_ARGS.append(opt)
             osx_console = True
         elif opt in ('--ipv6_hosting',):
             ipv6_hosting = arg
-            
+
     sabnzbd.MY_FULLNAME = os.path.normpath(os.path.abspath(sabnzbd.MY_FULLNAME))
     sabnzbd.MY_NAME = os.path.basename(sabnzbd.MY_FULLNAME)
     sabnzbd.DIR_PROG = os.path.dirname(sabnzbd.MY_FULLNAME)
@@ -1091,13 +1035,13 @@ def main():
     if sabnzbd.DAEMON:
         if enable_https and https_port:
             try:
-                cherrypy.process.servers.check_port(cherryhost, https_port)
+                cherrypy.process.servers.check_port(cherryhost, https_port, timeout=0.025)
             except IOError, error:
                 Bail_Out(browserhost, cherryport)
             except:
                 Bail_Out(browserhost, cherryport, '49')
         try:
-            cherrypy.process.servers.check_port(cherryhost, cherryport)
+            cherrypy.process.servers.check_port(cherryhost, cherryport, timeout=0.025)
         except IOError, error:
             Bail_Out(browserhost, cherryport)
         except:
@@ -1113,50 +1057,54 @@ def main():
     # If an instance of sabnzbd(same version) is already running on this port, launch the browser
     # If another program or sabnzbd version is on this port, try 10 other ports going up in a step of 5
     # If 'Port is not bound' (firewall) do not do anything (let the script further down deal with that).
+    notify_port_change = False
 
     # SSL
     if enable_https:
         port = https_port or cherryport
         try:
-            cherrypy.process.servers.check_port(browserhost, port)
+            cherrypy.process.servers.check_port(browserhost, port, timeout=0.025)
         except IOError, error:
             if str(error) == 'Port not bound.':
                 pass
             else:
                 if not url:
                     url = 'https://%s:%s/sabnzbd/api?' % (browserhost, port)
-                if new_instance or not check_for_sabnzbd(url, upload_nzbs, autobrowser):
-                    newport = find_free_port(browserhost, port)
-                    if newport > 0:
-                        sabnzbd.cfg.https_port.set(newport)
-                        if https_port:
-                            https_port = newport
-                        else:
-                            http_port = newport
+                if not sabnzbd.cfg.fixed_ports():
+                    if new_instance or not check_for_sabnzbd(url, upload_nzbs, autobrowser):
+                        newport = find_free_port(browserhost, port)
+                        if newport > 0:
+                            notify_port_change = True
+                            # Save the new port
+                            if https_port:
+                                https_port = newport
+                                sabnzbd.cfg.https_port.set(newport)
+                            else:
+                                # In case HTTPS == HTTP port
+                                http_port = newport
+                                sabnzbd.cfg.port.set(newport)
         except:
             Bail_Out(browserhost, cherryport, '49')
 
-    # NonSSL
-    try:
-        cherrypy.process.servers.check_port(browserhost, cherryport)
-    except IOError, error:
-        if str(error) == 'Port not bound.':
-            pass
-        else:
-            if not url:
-                url = 'http://%s:%s/sabnzbd/api?' % (browserhost, cherryport)
-            if new_instance or not check_for_sabnzbd(url, upload_nzbs, autobrowser):
-                port = find_free_port(browserhost, cherryport)
-                if port > 0:
-                    sabnzbd.cfg.cherryport.set(port)
-                    cherryport = port
-    except:
-        Bail_Out(browserhost, cherryport, '49')
-
-    if cherrypylogging is None:
-        cherrypylogging = sabnzbd.cfg.log_web()
-    else:
-        sabnzbd.cfg.log_web.set(cherrypylogging)
+    # NonSSL check if there's no HTTPS or we only use 1 port
+    if not (enable_https and not https_port):
+        try:
+            cherrypy.process.servers.check_port(browserhost, cherryport, timeout=0.025)
+        except IOError, error:
+            if str(error) == 'Port not bound.':
+                pass
+            else:
+                if not url:
+                    url = 'http://%s:%s/sabnzbd/api?' % (browserhost, cherryport)
+                if not sabnzbd.cfg.fixed_ports():
+                    if new_instance or not check_for_sabnzbd(url, upload_nzbs, autobrowser):
+                        port = find_free_port(browserhost, cherryport)
+                        if port > 0:
+                            sabnzbd.cfg.cherryport.set(port)
+                            notify_port_change = True
+                            cherryport = port
+        except:
+            Bail_Out(browserhost, cherryport, '49')
 
     if logging_level is None:
         logging_level = sabnzbd.cfg.log_level()
@@ -1181,19 +1129,12 @@ def main():
     # Prevent the logger from raising exceptions
     # primarily to reduce the fallout of Python issue 4749
     logging.raiseExceptions = 0
-
-    log_new = sabnzbd.cfg.log_new()
-    if log_new:
-        log_handler = NewRotatingFileHandler
-    else:
-        log_handler = logging.handlers.RotatingFileHandler
     sabnzbd.LOGFILE = os.path.join(logdir, DEF_LOG_FILE)
-    logsize = sabnzbd.cfg.log_size.get_int()
 
     try:
-        rollover_log = log_handler(
+        rollover_log = logging.handlers.RotatingFileHandler(
             sabnzbd.LOGFILE, 'a+',
-            logsize,
+            sabnzbd.cfg.log_size.get_int(),
             sabnzbd.cfg.log_backups())
 
         logformat = '%(asctime)s::%(levelname)s::[%(module)s:%(lineno)d] %(message)s'
@@ -1301,7 +1242,7 @@ def main():
         try:
             from ctypes import cdll
             libc = cdll.LoadLibrary('/usr/lib/libc.dylib')
-            boolSetResult = libc.setiopolicy_np(0, 1, 3)  # @UnusedVariable
+            boolSetResult = libc.setiopolicy_np(0, 1, 3)
             logging.info('[osx] IO priority set to throttle for process scope')
         except:
             logging.info('[osx] IO priority setting not supported')
@@ -1317,12 +1258,7 @@ def main():
         signal.signal(signal.SIGINT, sabnzbd.sig_handler)
         signal.signal(signal.SIGTERM, sabnzbd.sig_handler)
 
-    init_ok = sabnzbd.initialize(pause, clean_up, evalSched=True, repair=repair)
-
-    if not init_ok:
-        logging.error(T('Initializing %s-%s failed, aborting'),
-                      sabnzbd.MY_NAME, sabnzbd.__version__)
-        exit_sab(2)
+    sabnzbd.initialize(pause, clean_up, evalSched=True, repair=repair)
 
     os.chdir(sabnzbd.DIR_PROG)
 
@@ -1331,7 +1267,6 @@ def main():
     web_dirc = Web_Template(None, DEF_STDCONFIG, '')
 
     wizard_dir = os.path.join(sabnzbd.DIR_INTERFACES, 'wizard')
-    # sabnzbd.lang.install_language(os.path.join(wizard_dir, DEF_INT_LANGUAGE), sabnzbd.cfg.language(), 'wizard')
 
     sabnzbd.WEB_DIR = web_dir
     sabnzbd.WEB_DIR2 = web_dir2
@@ -1343,8 +1278,6 @@ def main():
     sabnzbd.WEB_COLOR2 = CheckColor(sabnzbd.cfg.web_color2(), web_dir2)
     sabnzbd.cfg.web_color2.set(sabnzbd.WEB_COLOR2)
 
-    logging.debug('Unwanted extensions are ... %s', sabnzbd.cfg.unwanted_extensions())
-
     if fork and not sabnzbd.WIN32:
         daemonize()
 
@@ -1368,11 +1301,8 @@ def main():
 
     print_modules()
 
-    import sabnzbd.utils.sslinfo
     logging.info("SSL version %s", sabnzbd.utils.sslinfo.ssl_version())
-    logging.info("pyOpenSSL version %s", sabnzbd.utils.sslinfo.pyopenssl_version())
-    logging.info("SSL potentially supported protocols %s", str(sabnzbd.utils.sslinfo.ssl_potential()))
-    logging.info("SSL actually supported protocols %s", str(sabnzbd.utils.sslinfo.ssl_protocols()))
+    logging.info("SSL supported protocols %s", str(sabnzbd.utils.sslinfo.ssl_protocols_labels()))
 
     cherrylogtoscreen = False
     sabnzbd.WEBLOGFILE = None
@@ -1381,7 +1311,7 @@ def main():
         if logdir:
             sabnzbd.WEBLOGFILE = os.path.join(logdir, DEF_LOG_CHERRY)
         # Define our custom logger for cherrypy errors
-        cherrypy_logging(sabnzbd.WEBLOGFILE, log_handler)
+        cherrypy_logging(sabnzbd.WEBLOGFILE, logging.handlers.RotatingFileHandler)
         if not fork:
             try:
                 x = sys.stderr.fileno
@@ -1415,7 +1345,7 @@ def main():
         hosts[1] = '::1'
 
     # The Windows binary requires numeric localhost as primary address
-    if multilocal and cherryhost == 'localhost':
+    if cherryhost == 'localhost':
         cherryhost = hosts[0]
 
     if enable_https:
@@ -1432,7 +1362,8 @@ def main():
             # Extra HTTPS port for secondary localhost
             attach_server(hosts[1], cherryport, https_cert, https_key)
 
-        cherrypy.config.update({'server.ssl_certificate': https_cert,
+        cherrypy.config.update({'server.ssl_module': 'builtin',
+                                'server.ssl_certificate': https_cert,
                                 'server.ssl_private_key': https_key,
                                 'server.ssl_certificate_chain': https_chain})
     elif multilocal:
@@ -1443,14 +1374,6 @@ def main():
         sabnzbd.cfg.username.set('')
         sabnzbd.cfg.password.set('')
 
-    # Fix leakage in memory-based CherryPy session support by using file-based.
-    # However, we don't really need session support.
-    if force_sessions:
-        sessions = sabnzbd.misc.create_real_path('sessions', sabnzbd.cfg.admin_dir.get_path(), 'sessions')[1]
-        sabnzbd.misc.remove_all(sessions, 'session-*.lock', keep_folder=True)
-    else:
-        sessions = None
-
     mime_gzip = ('text/*',
                  'application/javascript',
                  'application/x-javascript',
@@ -1463,17 +1386,12 @@ def main():
     cherrypy.config.update({'server.environment': 'production',
                             'server.socket_host': cherryhost,
                             'server.socket_port': cherryport,
+                            'server.shutdown_timeout': 0,
                             'log.screen': cherrylogtoscreen,
-                            'engine.autoreload.frequency': 100,
                             'engine.autoreload.on': False,
-                            'engine.reexec_retry': 100,
                             'tools.encode.on': True,
                             'tools.gzip.on': True,
                             'tools.gzip.mime_types': mime_gzip,
-                            'tools.sessions.on': bool(sessions),
-                            'tools.sessions.storage_type': 'file',
-                            'tools.sessions.storage_path': sessions,
-                            'tools.sessions.timeout': 60,
                             'request.show_tracebacks': True,
                             'checker.check_localhost': bool(consoleLogging),
                             'error_page.401': sabnzbd.panic.error_page_401,
@@ -1519,26 +1437,16 @@ def main():
     # Set authentication for CherryPy
     sabnzbd.interface.set_auth(cherrypy.config)
 
+    # Notify if port was changed
+    if notify_port_change:
+        logging.warning(T('Could not bind to configured port. Port changed to %s') % cherryport)
+
     logging.info('Starting web-interface on %s:%s', cherryhost, cherryport)
 
     sabnzbd.cfg.log_level.callback(guard_loglevel)
 
     try:
-        # Use internal cherrypy check first to prevent ugly tracebacks
-        cherrypy.process.servers.check_port(browserhost, cherryport)
         cherrypy.engine.start()
-    except IOError, error:
-        if str(error) == 'Port not bound.':
-            if not force_web:
-                panic_fwall(vista_plus)
-                sabnzbd.halt()
-                exit_sab(2)
-        else:
-            logging.error(T('Failed to start web-interface: '), exc_info=True)
-            Bail_Out(browserhost, cherryport, str(error))
-    except socket.error, error:
-        logging.error(T('Failed to start web-interface: '), exc_info=True)
-        Bail_Out(browserhost, cherryport)
     except:
         logging.error(T('Failed to start web-interface: '), exc_info=True)
         Bail_Out(browserhost, cherryport)
@@ -1547,26 +1455,6 @@ def main():
     cherrypy.engine.wait(cherrypy.process.wspbus.states.STARTED)
     sabnzbd.zconfig.set_bonjour(cherryhost, cherryport)
 
-    if enable_https:
-        browser_url = "https://%s:%s/sabnzbd" % (browserhost, cherryport)
-    else:
-        browser_url = "http://%s:%s/sabnzbd" % (browserhost, cherryport)
-    sabnzbd.BROWSER_URL = browser_url
-
-    if hasattr(cherrypy.wsgiserver, 'redirect_url'):
-        cherrypy.wsgiserver.redirect_url('https://%%s:%s/sabnzbd' % cherryport)
-
-    if not autorestarted:
-        launch_a_browser(browser_url)
-        if sabnzbd.FOUNDATION:
-            import sabnzbd.osxmenu
-            sabnzbd.osxmenu.notify("SAB_Launched", None)
-        growler.send_notification('SABnzbd%s' % growler.hostname(),
-                                  T('SABnzbd %s started') % sabnzbd.__version__, 'startup')
-        # Now's the time to check for a new version
-        check_latest_version()
-    autorestarted = False
-
     mail = None
     if sabnzbd.WIN32:
         if enable_https:
@@ -1604,17 +1492,35 @@ def main():
         for f in upload_nzbs:
             add_local(f)
 
+    # Set URL for browser
+    if enable_https:
+        browser_url = "https://%s:%s/sabnzbd" % (browserhost, cherryport)
+    else:
+        browser_url = "http://%s:%s/sabnzbd" % (browserhost, cherryport)
+    sabnzbd.BROWSER_URL = browser_url
+
+    if not autorestarted:
+        launch_a_browser(browser_url)
+        if sabnzbd.FOUNDATION:
+            import sabnzbd.osxmenu
+            sabnzbd.osxmenu.notify("SAB_Launched", None)
+        notifier.send_notification('SABnzbd%s' % notifier.hostname(),
+                                  T('SABnzbd %s started') % sabnzbd.__version__, 'startup')
+        # Now's the time to check for a new version
+        check_latest_version()
+    autorestarted = False
+
     # Have to keep this running, otherwise logging will terminate
-    timer = timer5 = 0
+    timer = 0
     while not sabnzbd.SABSTOP:
         if sabnzbd.LAST_WARNING:
             msg = sabnzbd.LAST_WARNING
             sabnzbd.LAST_WARNING = None
-            sabnzbd.growler.send_notification(T('Warning'), msg, 'warning')
+            sabnzbd.notifier.send_notification(T('Warning'), msg, 'warning')
         if sabnzbd.LAST_ERROR:
             msg = sabnzbd.LAST_ERROR
             sabnzbd.LAST_ERROR = None
-            sabnzbd.growler.send_notification(T('Error'), msg, 'error')
+            sabnzbd.notifier.send_notification(T('Error'), msg, 'error')
 
         if sabnzbd.WIN_SERVICE:
             rc = win32event.WaitForMultipleObjects((sabnzbd.WIN_SERVICE.hWaitStop,
@@ -1622,7 +1528,7 @@ def main():
             if rc == win32event.WAIT_OBJECT_0:
                 if mail:
                     mail.send('stop')
-                sabnzbd.save_state(flag=True)
+                sabnzbd.save_state()
                 logging.info('Leaving SABnzbd')
                 sabnzbd.SABSTOP = True
                 return
@@ -1649,58 +1555,40 @@ def main():
             # Check the threads
             if not sabnzbd.check_all_tasks():
                 autorestarted = True
-                cherrypy.engine.execv = True
+                sabnzbd.TRIGGER_RESTART = True
             # Notify guardian
             if sabnzbd.WIN_SERVICE and mail:
                 mail.send('active')
-
-            if timer5 > 9:
-                # 5 minute polling tasks
-                timer5 = 0
-                if sabnzbd.cfg.web_watchdog() and not is_sabnzbd_running('%s/api?tickleme=1' % sabnzbd.BROWSER_URL, 120):
-                    autorestarted = True
-                    cherrypy.engine.execv = True
-            else:
-                timer5 += 1
-
         else:
             timer += 1
 
         # 3 sec polling tasks
         # Check for auto-restart request
-        if cherrypy.engine.execv:
-            if sabnzbd.SCHED_RESTART:
-                scheduler.abort()
-                sabnzbd.halt()
-            else:
-                scheduler.stop()
-                sabnzbd.halt()
-                cherrypy.engine.exit()
+        # Or special restart cases like Mac and WindowsService
+        if sabnzbd.TRIGGER_RESTART:
+            # Shutdown
+            cherrypy.engine.exit()
+            sabnzbd.halt()
             sabnzbd.SABSTOP = True
+
             if sabnzbd.downloader.Downloader.do.paused:
-                re_argv.append('-p')
+                sabnzbd.RESTART_ARGS.append('-p')
             if autorestarted:
-                re_argv.append('--autorestarted')
-            sys.argv = re_argv
+                sabnzbd.RESTART_ARGS.append('--autorestarted')
+            sys.argv = sabnzbd.RESTART_ARGS
+
             os.chdir(org_dir)
-            if sabnzbd.DARWIN:
-                # TODO: when executing from sources on osx, after a restart, process is detached from console
-                # If OSX frozen restart of app instead of embedded python
-                if getattr(sys, 'frozen', None) == 'macosx_app':
-                    # [[NSProcessInfo processInfo] processIdentifier]]
-                    # logging.info("%s" % (NSProcessInfo.processInfo().processIdentifier()))
-                    my_pid = os.getpid()
-                    my_name = sabnzbd.MY_FULLNAME.replace('/Contents/MacOS/SABnzbd', '')
-                    my_args = ' '.join(sys.argv[1:])
-                    cmd = 'kill -9 %s && open "%s" --args %s' % (my_pid, my_name, my_args)
-                    logging.info('Launching: ', cmd)
-                    os.system(cmd)
-                else:
-                    args = sys.argv[:]
-                    args.insert(0, sys.executable)
-                    pid = os.fork()
-                    if pid == 0:
-                        os.execv(sys.executable, args)
+            # If OSX frozen restart of app instead of embedded python
+            if getattr(sys, 'frozen', None) == 'macosx_app':
+                # [[NSProcessInfo processInfo] processIdentifier]]
+                # logging.info("%s" % (NSProcessInfo.processInfo().processIdentifier()))
+                my_pid = os.getpid()
+                my_name = sabnzbd.MY_FULLNAME.replace('/Contents/MacOS/SABnzbd', '')
+                my_args = ' '.join(sys.argv[1:])
+                cmd = 'kill -9 %s && open "%s" --args %s' % (my_pid, my_name, my_args)
+                logging.info('Launching: ', cmd)
+                os.system(cmd)
+
             elif sabnzbd.WIN_SERVICE and mail:
                 logging.info('Asking the SABHelper service for a restart')
                 mail.send('restart')
@@ -1731,7 +1619,7 @@ def main():
             # Failing AppHelper libary!
             os._exit(0)
     else:
-        growler.send_notification('SABnzbd', T('SABnzbd shutdown finished'), 'startup')
+        notifier.send_notification('SABnzbd', T('SABnzbd shutdown finished'), 'startup')
         os._exit(0)
 
 
