@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,15 +22,16 @@ sabnzbd.bpsmeter - bpsmeter
 import time
 import logging
 import re
+from math import floor
 
 import sabnzbd
-from sabnzbd.constants import BYTES_FILE_NAME
+from sabnzbd.constants import BYTES_FILE_NAME, BYTES_FILE_NAME_OLD, KIBI
+from sabnzbd.encoding import unicoder
 import sabnzbd.cfg as cfg
 
-DAY = float(24*60*60)
+DAY = float(24 * 60 * 60)
 WEEK = DAY * 7
 
-#------------------------------------------------------------------------------
 
 def tomorrow(t):
     """ Return timestamp for tomorrow (midnight) """
@@ -63,12 +64,12 @@ def this_month(t):
 
 
 _DAYS = (0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
-def last_month_day(t=None):
+def last_month_day(tm):
     """ Return last day of this month """
-    t = t or time.localtime(t)
-    year, month = time.localtime(t)[:2]
+    year, month = tm[:2]
     day = _DAYS[month]
-    if day == 28 and (year % 4) == 0 and (year % 400) == 0:
+    # This simple formula for leap years is good enough
+    if day == 28 and (year % 4) == 0:
         day = 29
     return day
 
@@ -97,6 +98,21 @@ def next_month(t):
     return time.mktime(ntime)
 
 
+def fix_keys(data):
+    """ Convert keys of each dictionary in tuple 'data' to unicode """
+    new_data = []
+    if isinstance(data, list):
+        for n in xrange(len(data)):
+            if isinstance(data[n], dict):
+                new = {}
+                for key in data[n]:
+                    new[unicoder(key)] = data[n][key]
+            else:
+                new = data[n]
+            new_data.append(new)
+    return new_data
+
+
 class BPSMeter(object):
     do = None
 
@@ -104,47 +120,70 @@ class BPSMeter(object):
         t = time.time()
         self.start_time = t
         self.log_time = t
+        self.speed_log_time = t
         self.last_update = t
         self.bps = 0.0
+        self.bps_list = []
+        self.bps_list_max = 275
 
         self.day_total = {}
         self.week_total = {}
         self.month_total = {}
         self.grand_total = {}
 
-        self.end_of_day = tomorrow(t)     # Time that current day will end
-        self.end_of_week = next_week(t)   # Time that current day will end
-        self.end_of_month = next_month(t) # Time that current month will end
-        self.q_day = 1                    # Day of quota reset
-        self.q_period = 'm'               # Daily/Weekly/Monthly quota = d/w/m
-        self.quota = self.left = 0.0      # Quota and remaining quota
-        self.have_quota = False           # Flag for quota active
-        self.q_time = 0L            # Next reset time for quota
-        self.q_hour = 0                   # Quota reset hour
-        self.q_minute = 0                 # Quota reset minute
+        self.end_of_day = tomorrow(t)      # Time that current day will end
+        self.end_of_week = next_week(t)    # Time that current day will end
+        self.end_of_month = next_month(t)  # Time that current month will end
+        self.q_day = 1                     # Day of quota reset
+        self.q_period = 'm'                # Daily/Weekly/Monthly quota = d/w/m
+        self.quota = self.left = 0.0       # Quota and remaining quota
+        self.have_quota = False            # Flag for quota active
+        self.q_time = 0L                   # Next reset time for quota
+        self.q_hour = 0                    # Quota reset hour
+        self.q_minute = 0                  # Quota reset minute
+        self.quota_enabled = True          # Scheduled quota enable/disable
         BPSMeter.do = self
 
-
     def save(self):
         """ Save admin to disk """
-        if self.grand_total or self.day_total or self.week_total or self.month_total:
-            data = (self.last_update, self.grand_total,
-                    self.day_total, self.week_total, self.month_total,
-                    self.end_of_day, self.end_of_week, self.end_of_month,
-                    self.quota, self.left, self.q_time
-                   )
-            sabnzbd.save_admin(data, BYTES_FILE_NAME)
+        data = (self.last_update, self.grand_total,
+                self.day_total, self.week_total, self.month_total,
+                self.end_of_day, self.end_of_week, self.end_of_month,
+                self.quota, self.left, self.q_time
+               )
+        sabnzbd.save_admin(data, BYTES_FILE_NAME)
 
+    def defaults(self):
+        """ Get the latest data from the database and assign to a fake server """
+        logging.debug('Setting default BPS meter values')
+        history_db = sabnzbd.database.HistoryDB()
+        grand, month, week = history_db.get_history_size()
+        history_db.close()
+        self.grand_total = {}
+        self.month_total = {}
+        self.week_total = {}
+        self.day_total = {}
+        if grand:
+            self.grand_total['x'] = grand
+        if month:
+            self.month_total['x'] = month
+        if week:
+            self.week_total['x'] = week
+        self.quota = self.left = cfg.quota_size.get_float()
 
     def read(self):
-        """ Read admin from disk """
-        quota = self.left = cfg.quota_size.get_float() # Quota for this period
+        """ Read admin from disk, return True when pause is needed """
+        res = False
+        quota = self.left = cfg.quota_size.get_float()  # Quota for this period
         self.have_quota = bool(cfg.quota_size())
         data = sabnzbd.load_admin(BYTES_FILE_NAME)
+        if not data:
+            data = sabnzbd.load_admin(BYTES_FILE_NAME_OLD)
+            data = fix_keys(data)
         try:
             self.last_update, self.grand_total, \
-            self.day_total, self.week_total, self.month_total, \
-            self.end_of_day, self.end_of_week, self.end_of_month = data[:8]
+                self.day_total, self.week_total, self.month_total, \
+                self.end_of_day, self.end_of_week, self.end_of_month = data[:8]
             if len(data) == 11:
                 self.quota, self.left, self.q_time = data[8:]
                 logging.debug('Read quota q=%s l=%s reset=%s',
@@ -155,22 +194,18 @@ class BPSMeter(object):
                 self.quota = self.left = cfg.quota_size.get_float()
             res = self.reset_quota()
         except:
-            # Get the latest data from the database and assign to a fake server
-            logging.debug('Setting default BPS meter values')
-            grand, month, week  = sabnzbd.proxy_get_history_size()
-            if grand: self.grand_total['x'] = grand
-            if month: self.month_total['x'] = month
-            if week:  self.week_total['x'] = week
-            self.quota = self.left = cfg.quota_size.get_float()
-            res = False
-        # Force update of counters
-        self.update()
+            self.defaults()
+        # Force update of counters and validate data
+        try:
+            for server in self.grand_total.keys():
+                self.update(server)
+        except TypeError:
+            self.defaults()
+            self.update()
         return res
 
-
     def update(self, server=None, amount=0, testtime=None):
-        """ Update counters for "server" with "amount" bytes
-        """
+        """ Update counters for "server" with "amount" bytes """
         if testtime:
             t = testtime
         else:
@@ -206,18 +241,16 @@ class BPSMeter(object):
             self.grand_total[server] += amount
 
             # Quota check
-            if self.have_quota:
+            if self.have_quota and self.quota_enabled:
                 self.left -= amount
                 if self.left <= 0.0:
-                    from sabnzbd.downloader import Downloader
-                    if Downloader.do and not Downloader.do.paused:
-                        Downloader.do.pause()
-                        logging.warning(Ta('Quota spent, pausing downloading'))
+                    if sabnzbd.downloader.Downloader.do and not Downloader.do.paused:
+                        sabnzbd.downloader.Downloader.do.pause()
+                        logging.warning(T('Quota spent, pausing downloading'))
 
         # Speedometer
         try:
-            self.bps = (self.bps * (self.last_update - self.start_time)
-                        + amount) / (t - self.start_time)
+            self.bps = (self.bps * (self.last_update - self.start_time) + amount) / (t - self.start_time)
         except:
             self.bps = 0.0
 
@@ -235,6 +268,10 @@ class BPSMeter(object):
             logging.debug("bps: %s", self.bps)
             self.log_time = t
 
+        if self.speed_log_time < (t - 1.0):
+            self.add_empty_time()
+            self.bps_list.append(int(self.bps / KIBI))
+            self.speed_log_time = t
 
     def reset(self):
         t = time.time()
@@ -243,6 +280,14 @@ class BPSMeter(object):
         self.last_update = t
         self.bps = 0.0
 
+    def add_empty_time(self):
+        nr_diffs = int(time.time() - self.speed_log_time)
+        if nr_diffs > 1:
+            self.bps_list.extend([0] * nr_diffs)
+
+        if len(self.bps_list) > self.bps_list_max:
+            self.bps_list = self.bps_list[len(self.bps_list) - self.bps_list_max:]
+
     def get_sums(self):
         """ return tuple of grand, month, week, day totals """
         return (sum([v for v in self.grand_total.values()]),
@@ -260,14 +305,51 @@ class BPSMeter(object):
 
     def clear_server(self, server):
         """ Clean counters for specified server """
-        if server in self.day_total: del self.day_total[server]
-        if server in self.week_total : del self.week_total[server]
-        if server in self.month_total: del self.month_total[server]
-        if server in self.grand_total: del self.grand_total[server]
+        if server in self.day_total:
+            del self.day_total[server]
+        if server in self.week_total:
+            del self.week_total[server]
+        if server in self.month_total:
+            del self.month_total[server]
+        if server in self.grand_total:
+            del self.grand_total[server]
+        self.save()
 
     def get_bps(self):
         return self.bps
 
+    def get_bps_list(self):
+        refresh_rate = int(cfg.refresh_rate()) if cfg.refresh_rate() else 1
+        self.add_empty_time()
+        # We record every second, but display at the user's refresh-rate
+        return self.bps_list[::refresh_rate]
+
+    def get_stable_speed(self, timespan=10):
+        """ See if there is a stable speed the last <timespan> seconds
+            None: indicates it can't determine yet
+            False: the speed was not stable during <timespan>
+        """
+        if len(self.bps_list) < timespan:
+            return None
+
+        # Calculate the variance in the speed
+        avg = sum(self.bps_list[-timespan:])/timespan
+        vari = 0
+        for bps in self.bps_list[-timespan:]:
+            vari += abs(bps - avg)
+        vari = vari/timespan
+
+        try:
+            # See if the variance is less than 5%
+            if (vari / (self.bps/KIBI)) < 0.05:
+                return avg
+            else:
+                return False
+        except:
+            # Probably one of the values was 0
+            pass
+        return None
+
 
     def reset_quota(self, force=False):
         """ Check if it's time to reset the quota, optionally resuming
@@ -286,36 +368,34 @@ class BPSMeter(object):
             return True
 
     def next_reset(self, t=None):
-        """ Determine next reset time
-        """
+        """ Determine next reset time """
         t = t or time.time()
         tm = time.localtime(t)
         if self.q_period == 'd':
             nx = (tm[0], tm[1], tm[2], self.q_hour, self.q_minute, 0, 0, 0, tm[8])
-            if (tm.tm_hour + tm.tm_min * 60) >= (self.q_hour + self.q_minute * 60):
+            if (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute):
                 # If today's moment has passed, it will happen tomorrow
                 t = time.mktime(nx) + 24 * 3600
                 tm = time.localtime(t)
         elif self.q_period == 'w':
-            if self.q_day < tm.tm_wday+1 or (self.q_day == tm.tm_wday+1 and (tm.tm_hour + tm.tm_min * 60) >= (self.q_hour + self.q_minute * 60)):
+            if self.q_day < tm.tm_wday + 1 or (self.q_day == tm.tm_wday + 1 and (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute)):
                 tm = time.localtime(next_week(t))
             dif = abs(self.q_day - tm.tm_wday - 1)
             t = time.mktime(tm) + dif * 24 * 3600
             tm = time.localtime(t)
-        elif self.q_period ==  'm':
-            if self.q_day < tm.tm_mday or (self.q_day == tm.tm_mday and (tm.tm_hour + tm.tm_min * 60) >= (self.q_hour + self.q_minute * 60)):
+        elif self.q_period == 'm':
+            if self.q_day < tm.tm_mday or (self.q_day == tm.tm_mday and (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute)):
                 tm = time.localtime(next_month(t))
-            tm = (tm[0], tm[1], self.q_day, self.q_hour, self.q_minute, 0, 0, 0, tm[8])
+            day = min(last_month_day(tm), self.q_day)
+            tm = (tm[0], tm[1], day, self.q_hour, self.q_minute, 0, 0, 0, tm[8])
         else:
             return
         tm = (tm[0], tm[1], tm[2], self.q_hour, self.q_minute, 0, 0, 0, tm[8])
         self.q_time = time.mktime(tm)
         logging.debug('Will reset quota at %s', tm)
 
-
     def change_quota(self, allow_resume=True):
-        """ Update quota, potentially pausing downloader
-        """
+        """ Update quota, potentially pausing downloader """
         if not self.have_quota and self.quota < 0.5:
             # Never set, use last period's size
             per = cfg.quota_period()
@@ -330,24 +410,26 @@ class BPSMeter(object):
         self.have_quota = bool(cfg.quota_size())
         if self.have_quota:
             quota = cfg.quota_size.get_float()
-            self.left = quota - (self.quota - self.left)
+            if self.quota:
+                # Quota change, recalculate amount left
+                self.left = quota - (self.quota - self.left)
+            else:
+                # If previously no quota, self.left holds this period's usage
+                self.left = quota - self.left
             self.quota = quota
         else:
             self.quota = self.left = 0L
         self.update(0)
         self.next_reset()
-        if self.left > 0.5:
-            from sabnzbd.downloader import Downloader
-            if allow_resume and cfg.quota_resume() and Downloader.do and Downloader.do.paused:
-                Downloader.do.resume()
+        if self.left > 0.5 and allow_resume:
+            self.resume()
 
     # Pattern = <day#> <hh:mm>
     # The <day> and <hh:mm> part can both be optional
-    __re_day = re.compile('^\s*(\d+)[^:]*')
-    __re_hm = re.compile('(\d+):(\d+)\s*$')
+    __re_day = re.compile(r'^\s*(\d+)[^:]*')
+    __re_hm = re.compile(r'(\d+):(\d+)\s*$')
     def get_quota(self):
-        """ If quota active, return check-function, hour, minute
-        """
+        """ If quota active, return check-function, hour, minute """
         if self.have_quota:
             self.q_period = cfg.quota_period()[0].lower()
             self.q_day = 1
@@ -360,13 +442,35 @@ class BPSMeter(object):
             if m:
                 self.q_hour = int(m.group(1))
                 self.q_minute = int(m.group(2))
-            self.q_day = max(1, self.q_day)
-            self.q_day = min(7, self.q_day)
+            if self.q_period == 'w':
+                self.q_day = max(1, self.q_day)
+                self.q_day = min(7, self.q_day)
+            elif self.q_period == 'm':
+                self.q_day = max(1, self.q_day)
+                self.q_day = min(31, self.q_day)
+            else:
+                self.q_day = 1
             self.change_quota(allow_resume=False)
             return quota_handler, self.q_hour, self.q_minute
         else:
             return None, 0, 0
 
+    def set_status(self, status, action=True):
+        """ Disable/enable quota management """
+        self.quota_enabled = status
+        if action and not status:
+            self.resume()
+
+    def resume(self):
+        """ Resume downloading """
+        if cfg.quota_resume() and sabnzbd.downloader.Downloader.do and sabnzbd.downloader.Downloader.do.paused:
+            sabnzbd.downloader.Downloader.do.resume()
+
+    def midnight(self):
+        """ Midnight action: dummy update for all servers """
+        for server in self.day_total.keys():
+            self.update(server)
+
 
 def quota_handler():
     """ To be called from scheduler """
@@ -374,4 +478,8 @@ def quota_handler():
     BPSMeter.do.reset_quota()
 
 
+def midnight_action():
+    if BPSMeter.do:
+        BPSMeter.do.midnight()
+
 BPSMeter()
