@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,19 +21,18 @@ sabnzbd.cfg - Configuration Parameters
 import re
 
 import sabnzbd
-from sabnzbd.constants import DEF_HOST, DEF_PORT_WIN_SSL, DEF_PORT_WIN, DEF_STDINTF, \
-    DEF_DOWNLOAD_DIR, DEF_NZBBACK_DIR, DEF_PORT_UNIX_SSL, \
-    NORMAL_PRIORITY, DEF_SCANRATE, DEF_PORT_UNIX, DEF_COMPLETE_DIR, \
-    DEF_ADMIN_DIR
+from sabnzbd.constants import DEF_HOST, DEF_PORT, DEF_STDINTF, DEF_ADMIN_DIR, \
+    DEF_DOWNLOAD_DIR, DEF_NZBBACK_DIR, DEF_SCANRATE, DEF_COMPLETE_DIR
+
 from sabnzbd.config import OptionBool, OptionNumber, OptionPassword, \
     OptionDir, OptionStr, OptionList, no_nonsense, \
-    validate_octal, validate_safedir, validate_dir_exists, \
+    validate_octal, validate_safedir, \
     create_api_key, validate_notempty
 
 ##############################################################################
 # Email validation support
 ##############################################################################
-RE_VAL = re.compile('[^@ ]+@[^.@ ]+\.[^.@ ]')
+RE_VAL = re.compile(r'[^@ ]+@[^.@ ]+\.[^.@ ]')
 def validate_email(value):
     global email_endjob, email_full, email_rss
     if email_endjob() or email_full() or email_rss():
@@ -82,8 +81,9 @@ replace_illegal = OptionBool('misc', 'replace_illegal', True)
 pre_script = OptionStr('misc', 'pre_script', 'None')
 script_can_fail = OptionBool('misc', 'script_can_fail', False)
 start_paused = OptionBool('misc', 'start_paused', False)
-enable_https_verification = OptionBool('misc', 'enable_https_verification', False)
+enable_https_verification = OptionBool('misc', 'enable_https_verification', True)
 selftest_host = OptionStr('misc', 'selftest_host', 'self-test.sabnzbd.org')
+ssl_ciphers = OptionStr('misc', 'ssl_ciphers', '')
 
 enable_unrar = OptionBool('misc', 'enable_unrar', True)
 enable_unzip = OptionBool('misc', 'enable_unzip', True)
@@ -106,7 +106,7 @@ par2_multicore = OptionBool('misc', 'par2_multicore', True)
 allow_64bit_tools = OptionBool('misc', 'allow_64bit_tools', True)
 allow_streaming = OptionBool('misc', 'allow_streaming', False)
 pre_check = OptionBool('misc', 'pre_check', False)
-fail_hopeless = OptionBool('misc', 'fail_hopeless', False)
+fail_hopeless_jobs = OptionBool('misc', 'fail_hopeless_jobs', True)
 req_completion_rate = OptionNumber('misc', 'req_completion_rate', 100.2, 100, 200)
 
 rating_enable = OptionBool('misc', 'rating_enable', False)
@@ -145,9 +145,10 @@ backup_for_duplicates = OptionBool('misc', 'backup_for_duplicates', True)
 ignore_samples = OptionBool('misc', 'ignore_samples', False)
 create_group_folders = OptionBool('misc', 'create_group_folders', False)
 auto_sort = OptionBool('misc', 'auto_sort', False)
+propagation_delay = OptionNumber('misc', 'propagation_delay', 0)
 folder_rename = OptionBool('misc', 'folder_rename', True)
 folder_max_length = OptionNumber('misc', 'folder_max_length', DEF_FOLDER_MAX, 20, 65000)
-pause_on_pwrar = OptionBool('misc', 'pause_on_pwrar', True)
+pause_on_pwrar = OptionNumber('misc', 'pause_on_pwrar', 1)
 prio_sort_list = OptionList('misc', 'prio_sort_list')
 enable_meta = OptionBool('misc', 'enable_meta', True)
 
@@ -187,7 +188,6 @@ complete_dir = OptionDir('misc', 'complete_dir', DEF_COMPLETE_DIR, create=False,
 script_dir = OptionDir('misc', 'script_dir', create=True, writable=False)
 nzb_backup_dir = OptionDir('misc', 'nzb_backup_dir', DEF_NZBBACK_DIR)
 admin_dir = OptionDir('misc', 'admin_dir', DEF_ADMIN_DIR, validation=validate_safedir)
-# log_dir = OptionDir('misc', 'log_dir', 'logs')
 dirscan_dir = OptionDir('misc', 'dirscan_dir', create=False)
 dirscan_speed = OptionNumber('misc', 'dirscan_speed', DEF_SCANRATE, 0, 3600)
 size_limit = OptionStr('misc', 'size_limit', '0')
@@ -199,17 +199,12 @@ sanitize_safe = OptionBool('misc', 'sanitize_safe', False)
 api_logging = OptionBool('misc', 'api_logging', True)
 
 cherryhost = OptionStr('misc', 'host', DEF_HOST)
-if sabnzbd.WIN32:
-    cherryport = OptionStr('misc', 'port', DEF_PORT_WIN)
-else:
-    cherryport = OptionStr('misc', 'port', DEF_PORT_UNIX)
-if sabnzbd.WIN32:
-    https_port = OptionStr('misc', 'https_port', DEF_PORT_WIN_SSL)
-else:
-    https_port = OptionStr('misc', 'https_port', DEF_PORT_UNIX_SSL)
+cherryport = OptionStr('misc', 'port', DEF_PORT)
+https_port = OptionStr('misc', 'https_port')
 
 username = OptionStr('misc', 'username')
 password = OptionPassword('misc', 'password')
+html_login = OptionBool('misc', 'html_login', True)
 login_realm = OptionStr('misc', 'login_realm', 'SABnzbd')
 bandwidth_perc = OptionNumber('misc', 'bandwidth_perc', 0, 0, 100)
 bandwidth_max = OptionStr('misc', 'bandwidth_max')
@@ -222,12 +217,11 @@ web_color = OptionStr('misc', 'web_color', '')
 web_color2 = OptionStr('misc', 'web_color2')
 cleanup_list = OptionList('misc', 'cleanup_list')
 warned_old_queue = OptionBool('misc', 'warned_old_queue9', False)
-notified_new_skin = OptionBool('misc', 'notified_new_skin', False)
+notified_new_skin = OptionNumber('misc', 'notified_new_skin', 0)
 
 unwanted_extensions = OptionList('misc', 'unwanted_extensions')
 action_on_unwanted_extensions = OptionNumber('misc', 'action_on_unwanted_extensions', 0)
 
-log_web = OptionBool('logging', 'enable_cherrypy_logging', False)
 log_dir = OptionDir('misc', 'log_dir', 'logs', validation=validate_notempty)
 log_level = OptionNumber('logging', 'log_level', 1, -1, 2)
 log_size = OptionStr('logging', 'max_log_size', '5242880')
@@ -240,24 +234,21 @@ https_chain = OptionDir('misc', 'https_chain', create=False)
 enable_https = OptionBool('misc', 'enable_https', False)
 
 language = OptionStr('misc', 'language', 'en')
-unpack_check = OptionBool('misc', 'unpack_check', True)
 no_penalties = OptionBool('misc', 'no_penalties', False)
 load_balancing = OptionNumber('misc', 'load_balancing', 2)
 ipv6_servers = OptionNumber('misc', 'ipv6_servers', 1, 0, 2)
 
-# Internal options, not saved in INI file
-debug_delay = OptionNumber('misc', 'debug_delay', 0, add=False)
-
 api_key = OptionStr('misc', 'api_key', create_api_key())
 nzb_key = OptionStr('misc', 'nzb_key', create_api_key())
 disable_key = OptionBool('misc', 'disable_api_key', False, protect=True)
 api_warnings = OptionBool('misc', 'api_warnings', True, protect=True)
 local_ranges = OptionList('misc', 'local_ranges', protect=True)
-inet_exposure = OptionNumber('misc', 'inet_exposure', 0, protect=True)  # 0=local-only, 1=nzb, 2=api, 3=full_api, 4=webui
+inet_exposure = OptionNumber('misc', 'inet_exposure', 0, protect=True)  # 0=local-only, 1=nzb, 2=api, 3=full_api, 4=webui, 5=webui with login for external
 max_art_tries = OptionNumber('misc', 'max_art_tries', 3, 2)
 max_art_opt = OptionBool('misc', 'max_art_opt', False)
 use_pickle = OptionBool('misc', 'use_pickle', False)
 ipv6_hosting = OptionBool('misc', 'ipv6_hosting', False)
+fixed_ports = OptionBool('misc', 'fixed_ports', False)
 
 # [ncenter]
 ncenter_enable = OptionBool('ncenter', 'ncenter_enable', sabnzbd.DARWIN_VERSION > 7)
@@ -267,6 +258,7 @@ ncenter_prio_pp = OptionBool('ncenter', 'ncenter_prio_pp', False)
 ncenter_prio_complete = OptionBool('ncenter', 'ncenter_prio_complete', True)
 ncenter_prio_failed = OptionBool('ncenter', 'ncenter_prio_failed', True)
 ncenter_prio_disk_full = OptionBool('ncenter', 'ncenter_prio_disk_full', True)
+ncenter_prio_new_login = OptionBool('ncenter', 'ncenter_prio_new_login', False)
 ncenter_prio_warning = OptionBool('ncenter', 'ncenter_prio_warning', False)
 ncenter_prio_error = OptionBool('ncenter', 'ncenter_prio_error', False)
 ncenter_prio_queue_done = OptionBool('ncenter', 'ncenter_prio_queue_done', True)
@@ -280,6 +272,7 @@ acenter_prio_pp = OptionBool('acenter', 'acenter_prio_pp', False)
 acenter_prio_complete = OptionBool('acenter', 'acenter_prio_complete', True)
 acenter_prio_failed = OptionBool('acenter', 'acenter_prio_failed', True)
 acenter_prio_disk_full = OptionBool('acenter', 'acenter_prio_disk_full', True)
+acenter_prio_new_login = OptionBool('acenter', 'acenter_prio_new_login', False)
 acenter_prio_warning = OptionBool('acenter', 'acenter_prio_warning', False)
 acenter_prio_error = OptionBool('acenter', 'acenter_prio_error', False)
 acenter_prio_queue_done = OptionBool('acenter', 'acenter_prio_queue_done', True)
@@ -293,6 +286,7 @@ ntfosd_prio_pp = OptionBool('ntfosd', 'ntfosd_prio_pp', False)
 ntfosd_prio_complete = OptionBool('ntfosd', 'ntfosd_prio_complete', True)
 ntfosd_prio_failed = OptionBool('ntfosd', 'ntfosd_prio_failed', True)
 ntfosd_prio_disk_full = OptionBool('ntfosd', 'ntfosd_prio_disk_full', True)
+ntfosd_prio_new_login = OptionBool('ntfosd', 'ntfosd_prio_new_login', False)
 ntfosd_prio_warning = OptionBool('ntfosd', 'ntfosd_prio_warning', False)
 ntfosd_prio_error = OptionBool('ntfosd', 'ntfosd_prio_error', False)
 ntfosd_prio_queue_done = OptionBool('ntfosd', 'ntfosd_prio_queue_done', True)
@@ -308,6 +302,7 @@ growl_prio_pp = OptionBool('growl', 'growl_prio_pp', False)
 growl_prio_complete = OptionBool('growl', 'growl_prio_complete', True)
 growl_prio_failed = OptionBool('growl', 'growl_prio_failed', True)
 growl_prio_disk_full = OptionBool('growl', 'growl_prio_disk_full', True)
+growl_prio_new_login = OptionBool('growl', 'growl_prio_new_login', False)
 growl_prio_warning = OptionBool('growl', 'growl_prio_warning', False)
 growl_prio_error = OptionBool('growl', 'growl_prio_error', False)
 growl_prio_queue_done = OptionBool('growl', 'growl_prio_queue_done', True)
@@ -322,6 +317,7 @@ prowl_prio_pp = OptionNumber('prowl', 'prowl_prio_pp', -3)
 prowl_prio_complete = OptionNumber('prowl', 'prowl_prio_complete', 0)
 prowl_prio_failed = OptionNumber('prowl', 'prowl_prio_failed', 1)
 prowl_prio_disk_full = OptionNumber('prowl', 'prowl_prio_disk_full', 1)
+prowl_prio_new_login = OptionNumber('prowl', 'prowl_prio_new_login', -3)
 prowl_prio_warning = OptionNumber('prowl', 'prowl_prio_warning', -3)
 prowl_prio_error = OptionNumber('prowl', 'prowl_prio_error', -3)
 prowl_prio_queue_done = OptionNumber('prowl', 'prowl_prio_queue_done', 0)
@@ -338,6 +334,7 @@ pushover_prio_pp = OptionNumber('pushover', 'pushover_prio_pp', -3)
 pushover_prio_complete = OptionNumber('pushover', 'pushover_prio_complete', -1)
 pushover_prio_failed = OptionNumber('pushover', 'pushover_prio_failed', -1)
 pushover_prio_disk_full = OptionNumber('pushover', 'pushover_prio_disk_full', 1)
+pushover_prio_new_login = OptionNumber('pushover', 'pushover_prio_new_login', -3)
 pushover_prio_warning = OptionNumber('pushover', 'pushover_prio_warning', 1)
 pushover_prio_error = OptionNumber('pushover', 'pushover_prio_error', 1)
 pushover_prio_queue_done = OptionNumber('pushover', 'pushover_prio_queue_done', -1)
@@ -353,11 +350,27 @@ pushbullet_prio_pp = OptionNumber('pushbullet', 'pushbullet_prio_pp', 0)
 pushbullet_prio_complete = OptionNumber('pushbullet', 'pushbullet_prio_complete', 1)
 pushbullet_prio_failed = OptionNumber('pushbullet', 'pushbullet_prio_failed', 1)
 pushbullet_prio_disk_full = OptionNumber('pushbullet', 'pushbullet_prio_disk_full', 1)
+pushbullet_prio_new_login = OptionNumber('pushbullet', 'pushbullet_prio_new_login', 0)
 pushbullet_prio_warning = OptionNumber('pushbullet', 'pushbullet_prio_warning', 0)
 pushbullet_prio_error = OptionNumber('pushbullet', 'pushbullet_prio_error', 0)
 pushbullet_prio_queue_done = OptionNumber('pushbullet', 'pushbullet_prio_queue_done', 0)
 pushbullet_prio_other = OptionNumber('pushbullet', 'pushbullet_prio_other', 0)
 
+# [nscript]
+nscript_enable = OptionBool('nscript', 'nscript_enable')
+nscript_script = OptionStr('nscript', 'nscript_script')
+nscript_parameters = OptionStr('nscript', 'nscript_parameters')
+nscript_prio_startup = OptionBool('nscript', 'nscript_prio_startup', True)
+nscript_prio_download = OptionBool('nscript', 'nscript_prio_download', False)
+nscript_prio_pp = OptionBool('nscript', 'nscript_prio_pp', False)
+nscript_prio_complete = OptionBool('nscript', 'nscript_prio_complete', True)
+nscript_prio_failed = OptionBool('nscript', 'nscript_prio_failed', True)
+nscript_prio_disk_full = OptionBool('nscript', 'nscript_prio_disk_full', True)
+nscript_prio_new_login = OptionBool('nscript', 'nscript_prio_new_login', False)
+nscript_prio_warning = OptionBool('nscript', 'nscript_prio_warning', False)
+nscript_prio_error = OptionBool('nscript', 'nscript_prio_error', False)
+nscript_prio_queue_done = OptionBool('nscript', 'nscript_prio_queue_done', True)
+nscript_prio_other = OptionBool('nscript', 'nscript_prio_other', False)
 
 quota_size = OptionStr('misc', 'quota_size')
 quota_day = OptionStr('misc', 'quota_day')
@@ -369,13 +382,14 @@ osx_speed = OptionBool('misc', 'osx_speed', True)
 keep_awake = OptionBool('misc', 'keep_awake', True)
 win_menu = OptionBool('misc', 'win_menu', True)
 allow_incomplete_nzb = OptionBool('misc', 'allow_incomplete_nzb', False)
+prospective_par_download = OptionBool('misc', 'prospective_par_download', True)
 marker_file = OptionStr('misc', 'nomedia_marker', '')
 wait_ext_drive = OptionNumber('misc', 'wait_ext_drive', 5, 1, 60)
 queue_limit = OptionNumber('misc', 'queue_limit', 20, 0)
 history_limit = OptionNumber('misc', 'history_limit', 10, 0)
 show_sysload = OptionNumber('misc', 'show_sysload', 2, 0, 2)
-web_watchdog = OptionBool('misc', 'web_watchdog', False)
 enable_bonjour = OptionBool('misc', 'enable_bonjour', True)
+allow_duplicate_files = OptionBool('misc', 'allow_duplicate_files', False)
 warn_dupl_jobs = OptionBool('misc', 'warn_dupl_jobs', True)
 new_nzb_on_failure = OptionBool('misc', 'new_nzb_on_failure', False)
 
