@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -35,8 +35,8 @@ SAVE_CONFIG_LOCK = threading.Lock()
 
 
 CFG = {}                    # Holds INI structure
-                            # uring re-write this variable is global allow
-                            # direct access to INI structure
+                            # during re-write this variable is global
+                            # to allow direct access to INI structure
 
 database = {}               # Holds the option dictionary
 
@@ -49,12 +49,12 @@ class Option(object):
 
     def __init__(self, section, keyword, default_val=None, add=True, protect=False):
         """ Basic option
-            section     : single section or comma-separated list of sections
-                          a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
-            keyword     : keyword in the (last) section
-            default_val : value returned when no value has been set
-            callback    : procedure to call when value is successfully changed
-            protect     : Do not allow setting via the API (specifically set_dict)
+            `section`     : single section or comma-separated list of sections
+                            a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
+            `keyword`     : keyword in the (last) section
+            `default_val` : value returned when no value has been set
+            `callback`    : procedure to call when value is successfully changed
+            `protect`     : Do not allow setting via the API (specifically set_dict)
         """
         self.__sections = section.split(',')
         self.__keyword = keyword
@@ -79,7 +79,7 @@ class Option(object):
 
     def get(self):
         """ Retrieve value field """
-        if self.__value != None:
+        if self.__value is not None:
             return self.__value
         else:
             return self.__default_val
@@ -103,8 +103,8 @@ class Option(object):
     def __set(self, value):
         """ Set new value, no validation """
         global modified
-        if (value != None):
-            if type(value) == type([]) or type(value) == type({}) or value != self.__value:
+        if value is not None:
+            if isinstance(value, list) or isinstance(value, dict) or value != self.__value:
                 self.__value = value
                 modified = True
                 if self.__callback:
@@ -134,11 +134,11 @@ class OptionNumber(Option):
         self.__minval = minval
         self.__maxval = maxval
         self.__validation = validation
-        self.__int = type(default_val) == type(0)
+        self.__int = isinstance(default_val, int)
 
     def set(self, value):
         """ set new value, limited by range """
-        if value != None:
+        if value is not None:
             try:
                 if self.__int:
                     value = int(value)
@@ -150,9 +150,9 @@ class OptionNumber(Option):
                 error, val = self.__validation(value)
                 self._Option__set(val)
             else:
-                if (self.__maxval != None) and value > self.__maxval:
+                if self.__maxval is not None and value > self.__maxval:
                     value = self.__maxval
-                elif (self.__minval != None) and value < self.__minval:
+                elif self.__minval is not None and value < self.__minval:
                     value = self.__minval
                 self._Option__set(value)
         return None
@@ -187,7 +187,7 @@ class OptionDir(Option):
 
     def get(self):
         """ Return value, corrected for platform """
-        if self._Option__value != None:
+        if self._Option__value is not None:
             p = self._Option__value
         else:
             p = self._Option__default_val
@@ -225,7 +225,7 @@ class OptionDir(Option):
             'create' means try to create (but don't set permanent create flag)
         """
         error = None
-        if value != None and (create or value != self.get()):
+        if value is not None and (create or value != self.get()):
             value = value.strip()
             if self.__validation:
                 error, value = self.__validation(self.__root, value, self._Option__default_val)
@@ -270,15 +270,10 @@ class OptionList(Option):
     def get_string(self):
         """ Return the list as a comma-separated string """
         lst = self.get()
-        if type(lst) == type(""):
+        if isinstance(lst, basestring):
             return lst
-        txt = ''
-        r = len(lst)
-        for n in xrange(r):
-            txt += lst[n]
-            if n < r - 1:
-                txt += ', '
-        return txt
+        else:
+            return ', '.join(lst)
 
 
 class OptionStr(Option):
@@ -338,14 +333,14 @@ class OptionPassword(Option):
 
     def set(self, pw):
         """ Set password, encode it """
-        if (pw != None and pw == '') or (pw and pw.strip('*')):
+        if (pw is not None and pw == '') or (pw and pw.strip('*')):
             self._Option__set(encode_password(pw))
         return None
 
 
 @synchronized(CONFIG_LOCK)
 def add_to_database(section, keyword, obj):
-    """ add object as secion/keyword to INI database """
+    """ add object as section/keyword to INI database """
     global database
     if section not in database:
         database[section] = {}
@@ -382,10 +377,10 @@ class ConfigServer(object):
         self.password = OptionPassword(name, 'password', '', add=False)
         self.connections = OptionNumber(name, 'connections', 1, 0, 100, add=False)
         self.ssl = OptionBool(name, 'ssl', False, add=False)
+        self.ssl_verify = OptionNumber(name, 'ssl_verify', 1, add=False) # 0=No, 1=Normal, 2=Strict (hostname verification)
         self.enable = OptionBool(name, 'enable', True, add=False)
         self.optional = OptionBool(name, 'optional', False, add=False)
         self.retention = OptionNumber(name, 'retention', add=False)
-        self.ssl_type = OptionStr(name, 'ssl_type', 't1', add=False)
         self.send_group = OptionBool(name, 'send_group', False, add=False)
         self.priority = OptionNumber(name, 'priority', 0, 0, 100, add=False)
         # 'fillserver' field only here in order to set a proper priority when converting
@@ -399,8 +394,7 @@ class ConfigServer(object):
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
         for kw in ('displayname', 'host', 'port', 'timeout', 'username', 'password', 'connections', 'fillserver',
-                   'ssl', 'ssl_type', 'send_group', 'enable', 'optional', 'retention', 'priority',
-                   'categories', 'notes'):
+                   'ssl', 'ssl_verify', 'send_group', 'enable', 'optional', 'retention', 'priority', 'categories', 'notes'):
             try:
                 value = values[kw]
             except KeyError:
@@ -425,10 +419,10 @@ class ConfigServer(object):
             dict['password'] = self.password()
         dict['connections'] = self.connections()
         dict['ssl'] = self.ssl()
+        dict['ssl_verify'] = self.ssl_verify()
         dict['enable'] = self.enable()
         dict['optional'] = self.optional()
         dict['retention'] = self.retention()
-        dict['ssl_type'] = self.ssl_type()
         dict['send_group'] = self.send_group()
         dict['priority'] = self.priority()
         dict['categories'] = self.categories()
@@ -454,6 +448,7 @@ class ConfigCat(object):
         self.__name = name
         name = 'categories,' + name
 
+        self.order = OptionNumber(name, 'order', 0, 0, 100, add=False)
         self.pp = OptionStr(name, 'pp', '', add=False)
         self.script = OptionStr(name, 'script', 'Default', add=False)
         self.dir = OptionDir(name, 'dir', add=False, create=False)
@@ -465,7 +460,7 @@ class ConfigCat(object):
 
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
-        for kw in ('pp', 'script', 'dir', 'newzbin', 'priority'):
+        for kw in ('order', 'pp', 'script', 'dir', 'newzbin', 'priority'):
             try:
                 value = values[kw]
             except KeyError:
@@ -477,6 +472,7 @@ class ConfigCat(object):
         """ Return a dictionary with all attributes """
         dict = {}
         dict['name'] = self.__name
+        dict['order'] = self.order()
         dict['pp'] = self.pp()
         dict['script'] = self.script()
         dict['dir'] = self.dir()
@@ -543,7 +539,7 @@ class OptionFilters(Option):
             val = values.get(kw)
             if val is not None:
                 val = values[kw]
-                if type(val) == type([]):
+                if isinstance(val, list):
                     filters.append(val)
                 else:
                     filters.append(listquote.simplelist(val))
@@ -563,7 +559,7 @@ class ConfigRSS(object):
         self.__name = name
         name = 'rss,' + name
 
-        self.uri = OptionStr(name, 'uri', add=False)
+        self.uri = OptionList(name, 'uri', add=False)
         self.cat = OptionStr(name, 'cat', add=False)
         self.pp = OptionStr(name, 'pp', '', add=False)
         self.script = OptionStr(name, 'script', add=False)
@@ -611,7 +607,7 @@ class ConfigRSS(object):
 
 
 def get_dconfig(section, keyword, nested=False):
-    """ Return a config values dictonary,
+    """ Return a config values dictionary,
         Single item or slices based on 'section', 'keyword'
     """
     data = {}
@@ -786,7 +782,7 @@ def _read_config(path, try_backup=False):
 def save_config(force=False):
     """ Update Setup file with current option values """
     global CFG, database, modified
-    assert isinstance(CFG, configobj.ConfigObj)
+    if 0: assert isinstance(CFG, configobj.ConfigObj)
 
     if not (modified or force):
         return True
@@ -916,9 +912,18 @@ def get_categories(cat=0):
     if 'categories' not in database:
         database['categories'] = {}
     cats = database['categories']
+
+    # Add Default categories
     if '*' not in cats:
         ConfigCat('*', {'pp': old_def('dirscan_opts', '3'), 'script': old_def('dirscan_script', 'None'),
                         'priority': old_def('dirscan_priority', NORMAL_PRIORITY)})
+        # Add some category suggestions
+        ConfigCat('movies', {})
+        ConfigCat('tv', {})
+        ConfigCat('audio', {})
+        ConfigCat('software', {})
+
+        # Save config for future use
         save_config(True)
     if not isinstance(cat, int):
         try:
@@ -928,6 +933,25 @@ def get_categories(cat=0):
     return cats
 
 
+def get_ordered_categories():
+    """ Return list-copy of categories section that's ordered
+        by user's ordering including Default-category
+    """
+    database_cats = get_categories()
+
+    # Transform to list and sort
+    categories = []
+    for cat in database_cats.keys():
+        if cat != '*':
+            categories.append(database_cats[cat].get_dict())
+
+    # Sort and add default * category
+    categories.sort(key=lambda cat: cat['order'])
+    categories.insert(0, database_cats['*'].get_dict())
+
+    return categories
+
+
 def define_rss():
     """ Define rss-feeds listed in the Setup file
         return a list of ConfigRSS instances
