@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,41 +15,23 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from collections import namedtuple
+from re import compile
+
 CONFIG_VERSION = 19
 
 QUEUE_VERSION = 10
 POSTPROC_QUEUE_VERSION = 2
 
 REC_RAR_VERSION = 500
-PNFO_REPAIR_FIELD = 0
-PNFO_UNPACK_FIELD = 1
-PNFO_DELETE_FIELD = 2
-PNFO_SCRIPT_FIELD = 3
-PNFO_NZO_ID_FIELD = 4
-PNFO_FILENAME_FIELD = 5
-PNFO_UNPACKSTRHT_FIELD = 6
-PNFO_MSGID_FIELD = 7
-PNFO_EXTRA_FIELD1 = 8
-PNFO_EXTRA_FIELD2 = 9
-PNFO_BYTES_LEFT_FIELD = 10
-PNFO_BYTES_FIELD = 11
-PNFO_AVG_DATE_FIELD = 12
-PNFO_FINISHED_FILES_FIELD = 13
-PNFO_ACTIVE_FILES_FIELD = 14
-PNFO_QUEUED_FILES_FIELD = 15
-PNFO_STATUS_FIELD = 16
-PNFO_PRIORITY_FIELD = 17
-PNFO_MISSING_FIELD = 18
 
-QNFO_BYTES_FIELD = 0
-QNFO_BYTES_LEFT_FIELD = 1
-QNFO_PNFO_LIST_FIELD = 2
-QNFO_Q_SIZE_LIST_FIELD = 3
-QNFO_Q_FULLSIZE_FIELD = 4
+PNFO = namedtuple('PNFO', 'repair unpack delete script nzo_id filename password '
+                          'unpackstrht msgid category url bytes_left bytes avg_stamp '
+                          'avg_date finished_files active_files queued_files status priority missing')
 
-ANFO_ARTICLE_SUM_FIELD = 0
-ANFO_CACHE_SIZE_FIELD = 1
-ANFO_CACHE_LIMIT_FIELD = 2
+QNFO = namedtuple('QNFO', 'bytes bytes_left bytes_left_previous_page list q_size_list q_fullsize')
+
+ANFO = namedtuple('ANFO', 'article_sum cache_size cache_limit')
 
 GIGI = float(2 ** 30)
 MEBI = float(2 ** 20)
@@ -62,7 +44,6 @@ QUEUE_FILE_NAME = QUEUE_FILE_TMPL % QUEUE_VERSION
 POSTPROC_QUEUE_FILE_NAME = 'postproc%s.sab' % POSTPROC_QUEUE_VERSION
 RSS_FILE_NAME = 'rss_data.sab'
 SCAN_FILE_NAME = 'watched_data2.sab'
-TERM_FLAG_FILE = 'running.sab'
 FUTURE_Q_FOLDER = 'future'
 JOB_ADMIN = '__ADMIN__'
 VERIFIED_FILE = '__verified__'
@@ -92,22 +73,20 @@ DEF_SKIN_COLORS = {'smpl': 'white', 'Glitter': 'Default', 'plush': 'gold'}
 DEF_MAIN_TMPL = 'templates/main.tmpl'
 DEF_INI_FILE = 'sabnzbd.ini'
 DEF_HOST = '127.0.0.1'
-DEF_PORT_WIN = 8080
-DEF_PORT_UNIX = 8080
-DEF_PORT_WIN_SSL = 9090
-DEF_PORT_UNIX_SSL = 9090
+DEF_PORT = 8080
 DEF_WORKDIR = 'sabnzbd'
 DEF_LOG_FILE = 'sabnzbd.log'
 DEF_LOG_ERRFILE = 'sabnzbd.error.log'
 DEF_LOG_CHERRY = 'cherrypy.log'
+DEF_CACHE_LIMIT = '450M'
 DEF_TIMEOUT = 60
 MIN_TIMEOUT = 10
 MAX_TIMEOUT = 200
 DEF_LOGLEVEL = 1
 DEF_SCANRATE = 5
 DEF_QRATE = 0
-MIN_DECODE_QUEUE = 5
 MAX_DECODE_QUEUE = 10
+LIMIT_DECODE_QUEUE = 100
 MAX_WARNINGS = 20
 MAX_WIN_DFOLDER = 60
 
@@ -128,16 +107,16 @@ VALID_ARCHIVES = ('.zip', '.rar', '.7z')
 IGNORED_FOLDERS = ('@eaDir', '.appleDouble')
 
 # (MATCHER, [EXTRA, MATCHERS])
-series_match = [(r'( [sS]|[\d]+)x(\d+)',  # 1x01
-                [r'^[-\.]+([sS]|[\d])+x(\d+)', r'^[-\.](\d+)']),
+series_match = [(compile(r'( [sS]|[\d]+)x(\d+)'),  # 1x01
+                [compile(r'^[-\.]+([sS]|[\d])+x(\d+)'), compile(r'^[-\.](\d+)')]),
 
-                (r'[Ss](\d+)[\.\-]?[Ee](\d+)',  # S01E01
-                [r'^[-\.]+[Ss](\d+)[\.\-]?[Ee](\d+)', r'^[-\.](\d+)']),
+                (compile(r'[Ss](\d+)[\.\-]?[Ee](\d+)'),  # S01E01
+                [compile(r'^[-\.]+[Ss](\d+)[\.\-]?[Ee](\d+)'), compile(r'^[-\.](\d+)')]),
 
-                (r'[ \-_\.](\d)(\d{2,2})[ \-_\.]',  # .101. / _101_ / etc.
+                (compile(r'[ \-_\.](\d)(\d{2,2})[ \-_\.]'),  # .101. / _101_ / etc.
                 []),
 
-                (r'[ \-_\.](\d)(\d{2,2})$',  # .101 at end of title
+                (compile(r'[ \-_\.](\d)(\d{2,2})$'),  # .101 at end of title
                 [])
                ]
 
@@ -165,5 +144,6 @@ class Status():
     RUNNING = 'Running'             # PP: User's post processing script is running
     VERIFYING = 'Verifying'         # PP: Job is being verified (by par2)
     DELETED = 'Deleted'             # Q:  Job has been deleted (and is almost gone)
+    PROP = 'Propagating'            # Q:  Delayed download
 
-NOTIFY_KEYS = ('startup', 'download', 'pp', 'complete', 'failed', 'queue_done', 'disk_full', 'warning', 'error', 'other')
+NOTIFY_KEYS = ('startup', 'download', 'pp', 'complete', 'failed', 'queue_done', 'disk_full', 'new_login', 'warning', 'error', 'other')
