@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@ import Queue
 import binascii
 import logging
 import re
+from time import sleep
 from threading import Thread
 try:
     import _yenc
@@ -32,28 +33,31 @@ except ImportError:
     HAVE_YENC = False
 
 import sabnzbd
-from sabnzbd.constants import MAX_DECODE_QUEUE, MIN_DECODE_QUEUE
+from sabnzbd.constants import Status, MAX_DECODE_QUEUE, LIMIT_DECODE_QUEUE
 from sabnzbd.articlecache import ArticleCache
 import sabnzbd.downloader
 import sabnzbd.cfg as cfg
-from sabnzbd.encoding import name_fixer
+from sabnzbd.encoding import yenc_name_fixer
+from sabnzbd.misc import match_str
 
-#-------------------------------------------------------------------------------
 
 class CrcError(Exception):
+
     def __init__(self, needcrc, gotcrc, data):
         Exception.__init__(self)
         self.needcrc = needcrc
         self.gotcrc = gotcrc
         self.data = data
 
+
 class BadYenc(Exception):
+
     def __init__(self):
         Exception.__init__(self)
 
-#-------------------------------------------------------------------------------
 
 class Decoder(Thread):
+
     def __init__(self, servers):
         Thread.__init__(self)
 
@@ -62,7 +66,10 @@ class Decoder(Thread):
 
     def decode(self, article, lines):
         self.queue.put((article, lines))
-        if self.queue.qsize() > MAX_DECODE_QUEUE:
+        # See if there's space left in cache, pause otherwise
+        # But do allow some articles to enter queue, in case of full cache
+        qsize = self.queue.qsize()
+        if (not ArticleCache.do.reserve_space(lines) and qsize > MAX_DECODE_QUEUE) or (qsize > LIMIT_DECODE_QUEUE):
             sabnzbd.downloader.Downloader.do.delay()
 
     def stop(self):
@@ -71,77 +78,117 @@ class Decoder(Thread):
     def run(self):
         from sabnzbd.nzbqueue import NzbQueue
         while 1:
+            # Sleep to allow decoder/assembler switching
+            sleep(0.001)
             art_tup = self.queue.get()
             if not art_tup:
                 break
 
-            if self.queue.qsize() < MIN_DECODE_QUEUE and sabnzbd.downloader.Downloader.do.delayed:
-                sabnzbd.downloader.Downloader.do.undelay()
-
             article, lines = art_tup
             nzf = article.nzf
             nzo = nzf.nzo
+            art_id = article.article
+            killed = False
+
+            # Check if the space that's now free can let us continue the queue?
+            qsize = self.queue.qsize()
+            if (ArticleCache.do.free_reserve_space(lines) or qsize < MAX_DECODE_QUEUE) and (qsize < LIMIT_DECODE_QUEUE) and sabnzbd.downloader.Downloader.do.delayed:
+                sabnzbd.downloader.Downloader.do.undelay()
 
             data = None
-
             register = True  # Finish article
-            found = True     # Article found (only relevant for precheck)
+            found = False    # Proper article found
+            logme = None
 
             if lines:
-                logme = None
                 try:
-                    if nzo.precheck and '223' in lines[0]:
-                        raise IndexError
+                    if nzo.precheck:
+                        raise BadYenc
                     register = True
-                    logging.debug("Decoding %s", article)
+                    logging.debug("Decoding %s", art_id)
 
                     data = decode(article, lines)
                     nzf.article_count += 1
+                    found = True
+
                 except IOError, e:
-                    logme = Ta('Decoding %s failed') % article
-                    logging.info(logme)
+                    logme = T('Decoding %s failed') % art_id
+                    logging.warning(logme)
+                    logging.info("Traceback: ", exc_info=True)
+
                     sabnzbd.downloader.Downloader.do.pause()
-
                     article.fetcher = None
-
                     NzbQueue.do.reset_try_lists(nzf, nzo)
+                    register = False
 
+                except MemoryError, e:
+                    logme = T('Decoder failure: Out of memory')
+                    logging.warning(logme)
+                    anfo = sabnzbd.articlecache.ArticleCache.do.cache_info()
+                    logging.info("Decoder-Queue: %d, Cache: %d, %d, %d", self.queue.qsize(), anfo.article_sum, anfo.cache_size, anfo.cache_limit)
+                    logging.info("Traceback: ", exc_info=True)
+
+                    sabnzbd.downloader.Downloader.do.pause()
+                    article.fetcher = None
+                    NzbQueue.do.reset_try_lists(nzf, nzo)
                     register = False
 
                 except CrcError, e:
-                    logme = Ta('CRC Error in %s (%s -> %s)') % (article, e.needcrc, e.gotcrc)
+                    logme = T('CRC Error in %s (%s -> %s)') % (art_id, e.needcrc, e.gotcrc)
                     logging.info(logme)
 
                     data = e.data
 
-                    if cfg.fail_on_crc():
+                except BadYenc:
+                    # Handles precheck and badly formed articles
+                    killed = False
+                    found = False
+                    if nzo.precheck and lines and lines[0].startswith('223 '):
+                        # STAT was used, so we only get a status code
+                        found = True
+                    else:
+                        # Examine headers (for precheck) or body (for download)
+                        # And look for DMCA clues (while skipping "X-" headers)
+                        for line in lines:
+                            lline = line.lower()
+                            if 'message-id:' in lline:
+                                found = True
+                            if not line.startswith('X-') and match_str(lline, ('dmca', 'removed', 'cancel', 'blocked')):
+                                killed = True
+                                break
+                    if killed:
+                        logme = 'Article removed from server (%s)'
+                        logging.info(logme, art_id)
+                    if nzo.precheck:
+                        if found and not killed:
+                            # Pre-check, proper article found, just register
+                            logging.debug('Server has article %s', art_id)
+                            register = True
+                    elif not killed and not found:
+                        logme = T('Badly formed yEnc article in %s') % art_id
+                        logging.info(logme)
+
+                    if not found or killed:
                         new_server_found = self.__search_new_server(article)
                         if new_server_found:
                             register = False
                             logme = None
 
-                except BadYenc, e:
-                    logme = Ta('Badly formed yEnc article in %s') % article
-                    logging.info(logme)
-
-                    if cfg.fail_on_crc():
-                        new_server_found = self.__search_new_server(article)
-                        if new_server_found:
-                            register = False
-                            logme = None
-
-                except IndexError:
-                    # Pre-check, article found, just register
-                    register = True
-
                 except:
-                    logme = Ta('Unknown Error while decoding %s') % article
+                    logme = T('Unknown Error while decoding %s') % art_id
                     logging.info(logme)
-                    logging.info("Traceback: ", exc_info = True)
-                    pass
+                    logging.info("Traceback: ", exc_info=True)
+
+                    new_server_found = self.__search_new_server(article)
+                    if new_server_found:
+                        register = False
+                        logme = None
 
                 if logme:
-                    article.nzf.nzo.inc_log('bad_art_log', logme)
+                    if killed:
+                        nzo.inc_log('killed_art_log', art_id)
+                    else:
+                        nzo.inc_log('bad_art_log', art_id)
 
             else:
                 new_server_found = self.__search_new_server(article)
@@ -150,6 +197,11 @@ class Decoder(Thread):
                 elif nzo.precheck:
                     found = False
 
+            if logme or not found:
+                # Add extra parfiles when there was a damaged article
+                if cfg.prospective_par_download() and nzo.extrapars:
+                    nzo.prospective_add(nzf)
+
             if data:
                 ArticleCache.do.save_article(article, data)
 
@@ -168,7 +220,7 @@ class Decoder(Thread):
 
         for server in self.servers:
             if server.active and not article.server_in_try_list(server):
-                if server.fillserver:
+                if not sabnzbd.highest_server(server):
                     fill_server_found = True
                 else:
                     new_server_found = True
@@ -183,62 +235,64 @@ class Decoder(Thread):
             article.fetcher = None
             article.tries = 0
 
-            ## Allow all servers to iterate over this nzo and nzf again ##
+            # Allow all servers to iterate over this nzo and nzf again
             NzbQueue.do.reset_try_lists(nzf, nzo)
 
             if sabnzbd.LOG_ALL:
                 logging.debug('%s => found at least one untested server', article)
 
         else:
-            msg = Ta('%s => missing from all servers, discarding') % article
+            msg = T('%s => missing from all servers, discarding') % article
             logging.info(msg)
             article.nzf.nzo.inc_log('missing_art_log', msg)
 
-
         return new_server_found
-#-------------------------------------------------------------------------------
+
 
 YDEC_TRANS = ''.join([chr((i + 256 - 42) % 256) for i in xrange(256)])
 def decode(article, data):
-    data = strip(data)
-    ## No point in continuing if we don't have any data left
+    # Filter out empty ones
+    data = filter(None, data)
+    # No point in continuing if we don't have any data left
     if data:
         nzf = article.nzf
         yenc, data = yCheck(data)
         ybegin, ypart, yend = yenc
         decoded_data = None
 
-        #Deal with non-yencoded posts
+        # Deal with non-yencoded posts
         if not ybegin:
             found = False
-            for i in xrange(10):
-                if data[i].startswith('begin '):
-                    nzf.filename = name_fixer(data[i].split(None, 2)[2])
-                    nzf.type = 'uu'
-                    found = True
-                    break
+            try:
+                for i in xrange(min(40, len(data))):
+                    if data[i].startswith('begin '):
+                        nzf.type = 'uu'
+                        found = True
+                        # Pause the job and show warning
+                        if nzf.nzo.status != Status.PAUSED:
+                            nzf.nzo.pause()
+                            msg = T('UUencode detected, only yEnc encoding is supported [%s]') % nzf.nzo.final_name
+                            logging.warning(msg)
+                        break
+            except IndexError:
+                raise BadYenc()
+
             if found:
-                for n in xrange(i+1):
-                    data.pop(0)
-            if data[-1] == 'end':
-                data.pop()
-                if data[-1] == '`':
-                    data.pop()
-
-            decoded_data = '\r\n'.join(data)
-
-        #Deal with yenc encoded posts
-        elif (ybegin and yend):
-            if 'name' in ybegin:
-                nzf.filename = name_fixer(ybegin['name'])
+                decoded_data = ''
             else:
-                logging.debug("Possible corrupt header detected " + \
-                              "=> ybegin: %s", ybegin)
+                raise BadYenc()
+
+        # Deal with yenc encoded posts
+        elif ybegin and yend:
+            if 'name' in ybegin:
+                nzf.filename = yenc_name_fixer(ybegin['name'])
+            else:
+                logging.debug("Possible corrupt header detected => ybegin: %s", ybegin)
             nzf.type = 'yenc'
             # Decode data
             if HAVE_YENC:
                 decoded_data, crc = _yenc.decode_string(''.join(data))[:2]
-                partcrc = '%08X' % ((crc ^ -1) & 2**32L - 1)
+                partcrc = '%08X' % ((crc ^ -1) & 2 ** 32L - 1)
             else:
                 data = ''.join(data)
                 for i in (0, 9, 10, 13, 27, 32, 46, 61):
@@ -246,7 +300,7 @@ def decode(article, data):
                     data = data.replace(j, chr(i))
                 decoded_data = data.translate(YDEC_TRANS)
                 crc = binascii.crc32(decoded_data)
-                partcrc = '%08X' % (crc & 2**32L - 1)
+                partcrc = '%08X' % (crc & 2 ** 32L - 1)
 
             if ypart:
                 crcname = 'pcrc32'
@@ -257,23 +311,23 @@ def decode(article, data):
                 _partcrc = '0' * (8 - len(yend[crcname])) + yend[crcname].upper()
             else:
                 _partcrc = None
-                logging.debug("Corrupt header detected " + \
-                              "=> yend: %s", yend)
+                logging.debug("Corrupt header detected => yend: %s", yend)
 
-            if not (_partcrc == partcrc):
+            if not _partcrc == partcrc:
                 raise CrcError(_partcrc, partcrc, decoded_data)
         else:
             raise BadYenc()
 
         return decoded_data
 
+
 def yCheck(data):
     ybegin = None
     ypart = None
     yend = None
 
-    ## Check head
-    for i in xrange(40):
+    # Check head
+    for i in xrange(min(40, len(data))):
         try:
             if data[i].startswith('=ybegin '):
                 splits = 3
@@ -284,17 +338,17 @@ def yCheck(data):
 
                 ybegin = ySplit(data[i], splits)
 
-                if data[i+1].startswith('=ypart '):
-                    ypart = ySplit(data[i+1])
-                    data = data[i+2:]
+                if data[i + 1].startswith('=ypart '):
+                    ypart = ySplit(data[i + 1])
+                    data = data[i + 2:]
                     break
                 else:
-                    data = data[i+1:]
+                    data = data[i + 1:]
                     break
         except IndexError:
             break
 
-    ## Check tail
+    # Check tail
     for i in xrange(-1, -11, -1):
         try:
             if data[i].startswith('=yend '):
@@ -308,7 +362,7 @@ def yCheck(data):
 
 # Example: =ybegin part=1 line=128 size=123 name=-=DUMMY=- abc.par
 YSPLIT_RE = re.compile(r'([a-zA-Z0-9]+)=')
-def ySplit(line, splits = None):
+def ySplit(line, splits=None):
     fields = {}
 
     if splits:
@@ -320,19 +374,7 @@ def ySplit(line, splits = None):
         return fields
 
     for i in range(0, len(parts), 2):
-        key, value = parts[i], parts[i+1]
+        key, value = parts[i], parts[i + 1]
         fields[key] = value.strip()
 
     return fields
-
-def strip(data):
-    while data and not data[0]:
-        data.pop(0)
-
-    while data and not data[-1]:
-        data.pop()
-
-    for i in xrange(len(data)):
-        if data[i][:2] == '..':
-            data[i] = data[i][1:]
-    return data
