@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,29 +22,28 @@ sabnzbd.dirscanner - Scanner for Watched Folder
 import os
 import time
 import logging
-import re
 import zipfile
 import gzip
+import bz2
 import threading
 
 import sabnzbd
-from sabnzbd.constants import *
-from sabnzbd.utils.rarfile import is_rarfile, RarFile
+from sabnzbd.constants import SCAN_FILE_NAME, VALID_ARCHIVES
+import sabnzbd.utils.rarfile as rarfile
+from sabnzbd.encoding import platform_encode
+from sabnzbd.newsunpack import is_sevenfile, SevenZip
 import sabnzbd.nzbstuff as nzbstuff
 import sabnzbd.misc as misc
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 
-
 def name_to_cat(fname, cat=None):
-    """
-        Retrieve category from file name, but only if "cat" is None.
-    """
+    """ Retrieve category from file name, but only if "cat" is None. """
     if cat is None and fname.startswith('{{'):
         n = fname.find('}}')
         if n > 2:
             cat = fname[2:n].strip()
-            fname = fname[n+2:].strip()
+            fname = fname[n + 2:].strip()
             logging.debug('Job %s has category %s', fname, cat)
 
     return fname, cat
@@ -52,14 +51,49 @@ def name_to_cat(fname, cat=None):
 
 def CompareStat(tup1, tup2):
     """ Test equality of two stat-tuples, content-related parts only """
-    if tup1.st_ino   != tup2.st_ino:   return False
-    if tup1.st_size  != tup2.st_size:  return False
-    if tup1.st_mtime != tup2.st_mtime: return False
-    if tup1.st_ctime != tup2.st_ctime: return False
+    if tup1.st_ino != tup2.st_ino:
+        return False
+    if tup1.st_size != tup2.st_size:
+        return False
+    if tup1.st_mtime != tup2.st_mtime:
+        return False
+    if tup1.st_ctime != tup2.st_ctime:
+        return False
     return True
 
 
-def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False, priority=None, url=''):
+def is_archive(path):
+    """ Check if file in path is an ZIP, RAR or 7z file
+    :param path: path to file
+    :return: (zf, status, expected_extension)
+            status: -1==Error/Retry, 0==OK, 1==Ignore
+    """
+    if zipfile.is_zipfile(path):
+        try:
+            zf = zipfile.ZipFile(path)
+            return 0, zf, '.zip'
+        except:
+            return -1, None, ''
+    elif rarfile.is_rarfile(path):
+        try:
+            zf = rarfile.RarFile(path)
+            # Set path to tool to open it
+            rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
+            return 0, zf, '.rar'
+        except:
+            return -1, None, ''
+    elif is_sevenfile(path):
+        try:
+            zf = SevenZip(path)
+            return 0, zf, '.7z'
+        except:
+            return -1, None, ''
+    else:
+        return 1, None, ''
+
+
+def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
+                       priority=None, url='', nzbname=None, password=None, nzo_id=None):
     """ Analyse ZIP file and create job(s).
         Accepts ZIP files with ONLY nzb/nfo/folder files in it.
         returns (status, nzo_ids)
@@ -72,30 +106,23 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
 
     filename, cat = name_to_cat(filename, catdir)
 
-    if zipfile.is_zipfile(path):
-        try:
-            zf = zipfile.ZipFile(path)
-        except:
-            return -1, []
-    elif is_rarfile(path):
-        try:
-            zf = RarFile(path)
-        except:
-            return -1, []
-    else:
-        return 1, []
+    status, zf, extension = is_archive(path)
+
+    if status != 0:
+        return status, []
 
     status = 1
     names = zf.namelist()
-    names.sort()
+    nzbcount = 0
     for name in names:
         name = name.lower()
-        if not (name.endswith('.nzb') or name.endswith('.nfo') or name.endswith('/')):
-            status = 1
-            break
-        elif name.endswith('.nzb'):
+        if name.endswith('.nzb'):
             status = 0
+            nzbcount += 1
+
     if status == 0:
+        if nzbcount != 1:
+            nzbname = None
         for name in names:
             if name.lower().endswith('.nzb'):
                 try:
@@ -103,22 +130,30 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
                 except:
                     zf.close()
                     return -1, []
-                name = re.sub(r'\[.*nzbmatrix.com\]', '', name)
                 name = os.path.basename(name)
-                name = misc.sanitize_foldername(name)
                 if data:
                     try:
-                        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, url=url, priority=priority)
+                        nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, url=url,
+                                                 priority=priority, nzbname=nzbname)
+                        if not nzo.password:
+                            nzo.password = password
                     except:
                         nzo = None
                     if nzo:
+                        if nzo_id:
+                            # Re-use existing nzo_id, when a "future" job gets it payload
+                            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
+                            nzo.nzo_id = nzo_id
+                            nzo_id = None
                         nzo_ids.append(add_nzo(nzo))
+                        nzo.update_rating()
         zf.close()
         try:
-            if not keep: os.remove(path)
+            if not keep:
+                os.remove(path)
         except:
-            logging.error(Ta('Error removing %s'), path)
-            logging.info("Traceback: ", exc_info = True)
+            logging.error(T('Error removing %s'), misc.clip_path(path))
+            logging.info("Traceback: ", exc_info=True)
             status = 1
     else:
         zf.close()
@@ -128,9 +163,10 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
 
 
 def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
-                      priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url=''):
-    """ Analyse file and create a job from it
-        Supports NZB, NZB.GZ and GZ.NZB-in-disguise
+                      priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url='',
+                      password=None, nzo_id=None):
+    """ Analyze file and create a job from it
+        Supports NZB, NZB.BZ2, NZB.GZ and GZ.NZB-in-disguise
         returns (status, nzo_ids)
             status: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
     """
@@ -145,32 +181,44 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         b2 = f.read(1)
         f.close()
 
-        if (b1 == '\x1f' and b2 == '\x8b'):
+        if b1 == '\x1f' and b2 == '\x8b':
             # gzip file or gzip in disguise
             name = filename.replace('.nzb.gz', '.nzb')
             f = gzip.GzipFile(path, 'rb')
+        elif b1 == 'B' and b2 == 'Z':
+            # bz2 file or bz2 in disguise
+            name = filename.replace('.nzb.bz2', '.nzb')
+            f = bz2.BZ2File(path, 'rb')
         else:
             name = filename
             f = open(path, 'rb')
         data = f.read()
         f.close()
     except:
-        logging.warning(Ta('Cannot read %s'), path)
-        logging.info("Traceback: ", exc_info = True)
+        logging.warning(T('Cannot read %s'), misc.clip_path(path))
+        logging.info("Traceback: ", exc_info=True)
         return -2, nzo_ids
 
-
     if name:
         name, cat = name_to_cat(name, catdir)
         # The name is used as the name of the folder, so sanitize it using folder specific santization
-        name = misc.sanitize_foldername(name)
+        if not nzbname:
+            # Prevent embedded password from being damaged by sanitize and trimming
+            nzbname = os.path.split(name)[1]
 
     try:
-        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, priority=priority, nzbname=nzbname,
+        nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, priority=priority, nzbname=nzbname,
                                  nzo_info=nzo_info, url=url, reuse=reuse, dup_check=dup_check)
+        if not nzo.password:
+            nzo.password = password
     except TypeError:
         # Duplicate, ignore
+        if nzo_id:
+            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
         nzo = None
+    except ValueError:
+        # Empty, but correct file
+        return -1, nzo_ids
     except:
         if data.find("<nzb") >= 0 and data.find("</nzb") < 0:
             # Looks like an incomplete file, retry
@@ -179,12 +227,18 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             return -1, nzo_ids
 
     if nzo:
-        nzo_ids.append(add_nzo(nzo))
+        if nzo_id:
+            # Re-use existing nzo_id, when a "future" job gets it payload
+            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
+            nzo.nzo_id = nzo_id
+        nzo_ids.append(add_nzo(nzo, quiet=reuse))
+        nzo.update_rating()
     try:
-        if not keep: os.remove(path)
+        if not keep:
+            os.remove(path)
     except:
-        logging.error(Ta('Error removing %s'), path)
-        logging.info("Traceback: ", exc_info = True)
+        logging.error(T('Error removing %s'), misc.clip_path(path))
+        logging.info("Traceback: ", exc_info=True)
         return 1, nzo_ids
 
     return 0, nzo_ids
@@ -204,15 +258,13 @@ def CleanList(list, folder, files):
                 del list[path]
 
 
-#------------------------------------------------------------------------------
 class DirScanner(threading.Thread):
+    """ Thread that periodically scans a given directory and picks up any
+        valid NZB, NZB.GZ ZIP-with-only-NZB and even NZB.GZ named as .NZB
+        Candidates which turned out wrong, will be remembered and skipped in
+        subsequent scans, unless changed.
     """
-    Thread that periodically scans a given directoty and picks up any
-    valid NZB, NZB.GZ ZIP-with-only-NZB and even NZB.GZ named as .NZB
-    Candidates which turned out wrong, will be remembered and skipped in
-    subsequent scans, unless changed.
-    """
-    do = None # Access to instance of DirScanner
+    do = None  # Access to instance of DirScanner
 
     def __init__(self):
         threading.Thread.__init__(self)
@@ -225,11 +277,11 @@ class DirScanner(threading.Thread):
                 self.suspected = {}
         except:
             self.ignored = {}   # Will hold all unusable files and the
-                                # successfully processed ones that cannot be deleted
-            self.suspected = {} # Will hold name/attributes of suspected candidates
+            # successfully processed ones that cannot be deleted
+            self.suspected = {}  # Will hold name/attributes of suspected candidates
 
         self.shutdown = False
-        self.error_reported = False # Prevents mulitple reporting of missing watched folder
+        self.error_reported = False  # Prevents mulitple reporting of missing watched folder
         self.dirscan_dir = cfg.dirscan_dir.get_path()
         self.dirscan_speed = cfg.dirscan_speed()
         self.busy = False
@@ -252,13 +304,12 @@ class DirScanner(threading.Thread):
 
     def stop(self):
         """ Stop the dir scanner """
-        self.save()
         logging.info('Dirscanner shutting down')
         self.shutdown = True
 
     def save(self):
         """ Save dir scanner bookkeeping """
-        sabnzbd.save_admin((self.dirscan_dir, self.ignored, self.suspected), sabnzbd.SCAN_FILE_NAME)
+        sabnzbd.save_admin((self.dirscan_dir, self.ignored, self.suspected), SCAN_FILE_NAME)
 
     def run(self):
         """ Start the scanner """
@@ -283,17 +334,17 @@ class DirScanner(threading.Thread):
                 files = os.listdir(folder)
             except:
                 if not self.error_reported and not catdir:
-                    logging.error(Ta('Cannot read Watched Folder %s'), folder)
+                    logging.error(T('Cannot read Watched Folder %s'), misc.clip_path(folder))
                     self.error_reported = True
                 files = []
 
             for filename in files:
-                path = os.path.join(folder, filename)
-                if os.path.isdir(path) or path in self.ignored:
+                path = os.path.join(folder, platform_encode(filename))
+                if os.path.isdir(path) or path in self.ignored or filename[0] == '.':
                     continue
 
                 ext = os.path.splitext(path)[1].lower()
-                candidate = ext in ('.nzb', '.zip', '.gz', '.rar')
+                candidate = ext in ('.nzb', '.gz', '.bz2') or ext in VALID_ARCHIVES
                 if candidate:
                     try:
                         stat_tuple = os.stat(path)
@@ -330,8 +381,8 @@ class DirScanner(threading.Thread):
                     if not stable:
                         continue
 
-                    # Handle ZIP files, but only when containing just NZB files
-                    if ext in ('.zip', '.rar') :
+                    # Handle archive files, but only when containing just NZB files
+                    if ext in VALID_ARCHIVES:
                         res, nzo_ids = ProcessArchiveFile(filename, path, catdir=catdir, url=path)
                         if res == -1:
                             self.suspected[path] = stat_tuple
@@ -340,8 +391,8 @@ class DirScanner(threading.Thread):
                         else:
                             self.ignored[path] = 1
 
-                    # Handle .nzb, .nzb.gz or gzip-disguised-as-nzb
-                    elif ext == '.nzb' or filename.lower().endswith('.nzb.gz'):
+                    # Handle .nzb, .nzb.gz or gzip-disguised-as-nzb or .bz2
+                    elif ext == '.nzb' or filename.lower().endswith('.nzb.gz') or filename.lower().endswith('.nzb.bz2'):
                         res, nzo_id = ProcessSingleFile(filename, path, catdir=catdir, url=path)
                         if res < 0:
                             self.suspected[path] = stat_tuple
@@ -366,13 +417,13 @@ class DirScanner(threading.Thread):
                     list = os.listdir(dirscan_dir)
                 except:
                     if not self.error_reported:
-                        logging.error(Ta('Cannot read Watched Folder %s'), dirscan_dir)
+                        logging.error(T('Cannot read Watched Folder %s'), misc.clip_path(dirscan_dir))
                         self.error_reported = True
                     list = []
 
                 cats = config.get_categories()
                 for dd in list:
-                    dpath = os.path.join(dirscan_dir, dd)
+                    dpath = os.path.join(dirscan_dir, platform_encode(dd))
                     if os.path.isdir(dpath) and dd.lower() in cats:
                         run_dir(dpath, dd.lower())
             self.busy = False
