@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,18 +29,15 @@ import threading
 import subprocess
 import socket
 import time
+import datetime
 import fnmatch
 import stat
-try:
-    socket.ssl
-    _HAVE_SSL = True
-except:
-    _HAVE_SSL = False
 from urlparse import urlparse
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, MEBI
+from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, \
+     GIGI, MEBI, DEF_CACHE_LIMIT
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import unicoder, special_fixer, gUTF
@@ -61,6 +58,40 @@ def time_format(fmt):
         return fmt
 
 
+def calc_age(date, trans=False):
+    """ Calculate the age difference between now and date.
+        Value is returned as either days, hours, or minutes.
+        When 'trans' is True, time symbols will be translated.
+    """
+    if trans:
+        d = T('d')  # : Single letter abbreviation of day
+        h = T('h')  # : Single letter abbreviation of hour
+        m = T('m')  # : Single letter abbreviation of minute
+    else:
+        d = 'd'
+        h = 'h'
+        m = 'm'
+    try:
+        now = datetime.datetime.now()
+        # age = str(now - date).split(".")[0] #old calc_age
+
+        # time difference
+        dage = now - date
+        seconds = dage.seconds
+        # only one value should be returned
+        # if it is less than 1 day then it returns in hours, unless it is less than one hour where it returns in minutes
+        if dage.days:
+            age = '%s%s' % (dage.days, d)
+        elif seconds / 3600:
+            age = '%s%s' % (seconds / 3600, h)
+        else:
+            age = '%s%s' % (seconds / 60, m)
+    except:
+        age = "-"
+
+    return age
+
+
 def safe_lower(txt):
     """ Return lowercased string. Return '' for None """
     if txt:
@@ -151,42 +182,41 @@ def wildcard_to_re(text):
 def cat_convert(cat):
     """ Convert indexer's category/group-name to user categories.
         If no match found, but indexer-cat equals user-cat, then return user-cat
+        If no match found, but the indexer-cat starts with the user-cat, return user-cat
         If no match found, return None
     """
-    newcat = cat
-    found = False
-
     if cat and cat.lower() != 'none':
-        cats = config.get_categories()
+        cats = config.get_ordered_categories()
+        raw_cats = config.get_categories()
         for ucat in cats:
             try:
-                indexer = cats[ucat].newzbin()
-                if type(indexer) != type([]):
+                # Ordered cat-list has tags only as string
+                indexer = raw_cats[ucat['name']].newzbin()
+                if not isinstance(indexer, list):
                     indexer = [indexer]
             except:
                 indexer = []
             for name in indexer:
                 if re.search('^%s$' % wildcard_to_re(name), cat, re.I):
-                    if '.' not in name:
-                        logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat)
+                    if '.' in name:
+                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat['name'])
                     else:
-                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat)
-                    newcat = ucat
-                    found = True
-                    break
-            if found:
-                break
+                        logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                    return ucat['name']
 
-        if not found:
-            for ucat in cats:
-                if cat.lower() == ucat.lower():
-                    found = True
-                    break
+        # Try to find full match between user category and indexer category
+        for ucat in cats:
+            if cat.lower() == ucat['name'].lower():
+                logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                return ucat['name']
 
-    if found:
-        return newcat
-    else:
-        return None
+        # Try to find partial match between user category and indexer category
+        for ucat in cats:
+            if cat.lower().startswith(ucat['name'].lower()):
+                logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                return ucat['name']
+
+    return None
 
 
 ##############################################################################
@@ -209,6 +239,17 @@ def replace_win_devices(name):
                 break
     return name
 
+
+def has_win_device(p):
+    """ Return True if filename part contains forbidden name
+    """
+    p = os.path.split(p)[1].lower()
+    for dev in _DEVICES:
+        if p == dev or p.startswith(dev + '.'):
+            return True
+    return False
+
+
 if sabnzbd.WIN32:
     # the colon should be here too, but we'll handle that separately
     CH_ILLEGAL = r'\/<>?*|"'
@@ -218,7 +259,7 @@ else:
     CH_LEGAL = r'+'
 
 
-def sanitize_filename(name):
+def sanitize_filename(name, allow_win_devices=False):
     """ Return filename with illegal chars converted to legal ones
         and with the par2 extension always in lowercase
     """
@@ -235,6 +276,9 @@ def sanitize_filename(name):
             # Compensate for the foolish way par2 on OSX handles a colon character
             name = name[name.rfind(':') + 1:]
 
+    if sabnzbd.WIN32 and not allow_win_devices:
+        name = replace_win_devices(name)
+
     lst = []
     for ch in name.strip():
         if ch in illegal:
@@ -329,6 +373,24 @@ def sanitize_and_trim_path(path):
     return os.path.abspath(os.path.normpath(new_path))
 
 
+def sanitize_files_in_folder(folder):
+    """ Sanitize each file in the folder, return list of new names
+    """
+    lst = []
+    for root, _, files in os.walk(folder):
+        for file_ in files:
+            path = os.path.join(root, file_)
+            new_path = os.path.join(root, sanitize_filename(file_))
+            if path != new_path:
+                try:
+                    os.rename(path, new_path)
+                    path = new_path
+                except:
+                    logging.debug('Cannot rename %s to %s', path, new_path)
+            lst.append(path)
+    return lst
+
+
 def flag_file(path, flag, create=False):
     """ Create verify flag file or return True if it already exists """
     path = os.path.join(path, JOB_ADMIN)
@@ -624,13 +686,13 @@ def check_latest_version():
     if testver and current < latest:
         # This is a test version, but user has't seen the
         # "Final" of this one yet, so show the Final
-        sabnzbd.NEW_VERSION = '%s;%s' % (latest_label, url)
+        sabnzbd.NEW_VERSION = (latest_label, url)
     elif current < latest:
         # This one is behind, show latest final
-        sabnzbd.NEW_VERSION = '%s;%s' % (latest_label, url)
+        sabnzbd.NEW_VERSION = (latest_label, url)
     elif testver and current < latest_test:
         # This is a test version beyond the latest Final, so show latest Alpha/Beta/RC
-        sabnzbd.NEW_VERSION = '%s;%s' % (latest_testlabel, url_beta)
+        sabnzbd.NEW_VERSION = (latest_testlabel, url_beta)
 
 
 def from_units(val):
@@ -760,6 +822,28 @@ def check_mount(path):
     return not m
 
 
+def get_cache_limit():
+    """ Depending on OS, calculate cache limit """
+    # OSX/Windows use Default value
+    if sabnzbd.WIN32 or sabnzbd.DARWIN:
+        return DEF_CACHE_LIMIT
+
+    # Calculate, if possible
+    try:
+        # Use 1/4th of available memory
+        mem_bytes = (os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES'))/4
+        # Not more than the maximum we think is reasonable
+        if mem_bytes > from_units(DEF_CACHE_LIMIT):
+            return DEF_CACHE_LIMIT
+        elif mem_bytes > from_units('32M'):
+            # We make sure it's at least a valid value
+            return to_units(mem_bytes)
+    except:
+        pass
+    # If failed, leave empty so user needs to decide
+    return ''
+
+
 ##############################################################################
 # Locked directory operations
 ##############################################################################
@@ -827,19 +911,24 @@ def move_to_path(path, new_path):
         new_path = get_unique_filename(new_path)
 
     if new_path:
-        logging.debug("Moving. Old path:%s new path:%s overwrite?:%s",
+        logging.debug("Moving. Old path: %s New path: %s Overwrite: %s",
                                                   path, new_path, overwrite)
         try:
             # First try cheap rename
             renamer(path, new_path)
         except:
             # Cannot rename, try copying
+            logging.debug("File could not be renamed, trying copying: %s", path)
             try:
                 if not os.path.exists(os.path.dirname(new_path)):
                     create_dirs(os.path.dirname(new_path))
                 shutil.copyfile(path, new_path)
                 os.remove(path)
             except:
+                # Check if the old-file actually exists (possible delete-delays)
+                if not os.path.exists(path):
+                    logging.debug("File not moved, original path gone: %s", path)
+                    return True, None
                 if not (cfg.marker_file() and cfg.marker_file() in path):
                     logging.error(T('Failed moving %s to %s'), clip_path(path), clip_path(new_path))
                     logging.info("Traceback: ", exc_info=True)
@@ -1074,83 +1163,75 @@ if sabnzbd.WIN32:
     except:
         pass
 
-    def diskfree(_dir):
-        """ Return amount of free diskspace in GBytes """
+    def diskspace_base(_dir):
+        """ Return amount of free and used diskspace in GBytes """
         _dir = find_dir(_dir)
         try:
             available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(_dir)
-            return available / GIGI
+            return disk_size / GIGI, available / GIGI
         except:
-            return 0.0
+            return 0.0, 0.0
 
-    def disktotal(_dir):
-        """ Return amount of free diskspace in GBytes """
-        _dir = find_dir(_dir)
-        try:
-            available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(_dir)
-            return disk_size / GIGI
-        except:
-            return 0.0
 else:
     try:
         os.statvfs
         # posix diskfree
-
-        def diskfree(_dir):
-            """ Return amount of free diskspace in GBytes """
-            _dir = find_dir(_dir)
-            try:
-                s = os.statvfs(_dir)
-                if s.f_bavail < 0:
-                    return float(sys.maxint) * float(s.f_frsize) / GIGI
-                else:
-                    return float(s.f_bavail) * float(s.f_frsize) / GIGI
-            except OSError:
-                return 0.0
-
-        def disktotal(_dir):
-            """ Return amount of total diskspace in GBytes """
+        def diskspace_base(_dir):
+            """ Return amount of free and used diskspace in GBytes """
             _dir = find_dir(_dir)
             try:
                 s = os.statvfs(_dir)
                 if s.f_blocks < 0:
-                    return float(sys.maxint) * float(s.f_frsize) / GIGI
+                    disk_size = float(sys.maxint) * float(s.f_frsize)
                 else:
-                    return float(s.f_blocks) * float(s.f_frsize) / GIGI
-            except OSError:
-                return 0.0
+                    disk_size = float(s.f_blocks) * float(s.f_frsize)
+                if s.f_bavail < 0:
+                    available = float(sys.maxint) * float(s.f_frsize)
+                else:
+                    available = float(s.f_bavail) * float(s.f_frsize)
+                return disk_size / GIGI, available / GIGI
+            except:
+                return 0.0, 0.0
     except ImportError:
-        def diskfree(_dir):
-            return 10.0
-
-        def disktotal(_dir):
-            return 20.0
+        def diskspace_base(_dir):
+            return 20.0, 10.0
 
 
+__LAST_DISK_RESULT = {}
+__LAST_DISK_CALL = {}
+def diskspace(_dir, force=False):
+    """ Wrapper to cache results """
+    if _dir not in __LAST_DISK_RESULT:
+        __LAST_DISK_RESULT[_dir] = [0.0, 0.0]
+        __LAST_DISK_CALL[_dir] = 0.0
+
+    # When forced, ignore any cache to avoid problems in UI
+    if force:
+        return diskspace_base(_dir)
+
+    # Check against cache
+    if time.time() > __LAST_DISK_CALL[_dir] + 10.0:
+        __LAST_DISK_RESULT[_dir] = diskspace_base(_dir)
+        __LAST_DISK_CALL[_dir] = time.time()
+
+    return __LAST_DISK_RESULT[_dir]
+
+
+##############################################################################
+# Other support functions
+##############################################################################
 def create_https_certificates(ssl_cert, ssl_key):
     """ Create self-signed HTTPS certificates and store in paths 'ssl_cert' and 'ssl_key' """
-    try:
-        from OpenSSL import crypto
-        from sabnzbd.utils.certgen import createKeyPair, createCertRequest, createCertificate, \
-             TYPE_RSA, serial
-    except:
-        logging.warning(T('pyopenssl module missing, please install for https access'))
+    if not sabnzbd.HAVE_CRYPTOGRAPHY:
+        logging.error(T('%s missing'), 'Python Cryptography')
         return False
 
-    # Create the CA Certificate
-    cakey = createKeyPair(TYPE_RSA, 1024)
-    careq = createCertRequest(cakey, CN='Certificate Authority')
-    cacert = createCertificate(careq, (careq, cakey), serial, (0, 60 * 60 * 24 * 365 * 10))  # ten years
-
-    cname = 'SABnzbd'
-    pkey = createKeyPair(TYPE_RSA, 1024)
-    req = createCertRequest(pkey, CN=cname)
-    cert = createCertificate(req, (cacert, cakey), serial, (0, 60 * 60 * 24 * 365 * 10))  # ten years
-
     # Save the key and certificate to disk
     try:
-        open(ssl_key, 'w').write(crypto.dump_privatekey(crypto.FILETYPE_PEM, pkey))
-        open(ssl_cert, 'w').write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
+        from sabnzbd.utils.certgen import generate_key, generate_local_cert
+        private_key = generate_key(key_size=2048, output_file=ssl_key)
+        cert = generate_local_cert(private_key, days_valid=3560, output_file=ssl_cert, LN=u'SABnzbd', ON=u'SABnzbd', CN=u'localhost')
+        logging.info('Self-signed certificates generated successfully')
     except:
         logging.error(T('Error creating SSL key and certificate'))
         logging.info("Traceback: ", exc_info=True)
@@ -1159,6 +1240,49 @@ def create_https_certificates(ssl_cert, ssl_key):
     return True
 
 
+def get_all_passwords(nzo):
+    """ Get all passwords, from the NZB, meta and password file """
+    if nzo.password:
+        logging.info('Found a password that was set by the user: %s', nzo.password)
+        passwords = [nzo.password.strip()]
+    else:
+        passwords = []
+
+    meta_passwords = nzo.meta.get('password', [])
+    pw = nzo.nzo_info.get('password')
+    if pw:
+        meta_passwords.append(pw)
+    if meta_passwords:
+        if nzo.password == meta_passwords[0]:
+            # this nzo.password came from meta, so don't use it twice
+            passwords.extend(meta_passwords[1:])
+        else:
+            passwords.extend(meta_passwords)
+        logging.info('Read %s passwords from meta data in NZB: %s', len(meta_passwords), meta_passwords)
+    pw_file = cfg.password_file.get_path()
+    if pw_file:
+        try:
+            pwf = open(pw_file, 'r')
+            lines = pwf.read().split('\n')
+            # Remove empty lines and space-only passwords and remove surrounding spaces
+            pws = [pw.strip('\r\n ') for pw in lines if pw.strip('\r\n ')]
+            logging.debug('Read these passwords from file: %s', pws)
+            passwords.extend(pws)
+            pwf.close()
+            logging.info('Read %s passwords from file %s', len(pws), pw_file)
+        except IOError:
+            logging.info('Failed to read the passwords file %s', pw_file)
+
+    if nzo.password:
+        # If an explicit password was set, add a retry without password, just in case.
+        passwords.append('')
+    elif not passwords or nzo.encrypted < 1:
+        # If we're not sure about encryption, start with empty password
+        # and make sure we have at least the empty password
+        passwords.insert(0, '')
+    return passwords
+
+
 def find_on_path(targets):
     """ Search the PATH for a program and return full path """
     if sabnzbd.WIN32:
@@ -1225,6 +1349,18 @@ def renamer(old, new):
     if sabnzbd.WIN32:
         retries = 15
         while retries > 0:
+            # First we try 3 times with os.rename
+            if retries > 12:
+                try:
+                    os.rename(old, new)
+                    return
+                except:
+                    retries -= 1
+                    time.sleep(3)
+                    continue
+
+            # Now we try the back-up method
+            logging.debug('Could not rename, trying move for %s to %s', old, new)
             try:
                 shutil.move(old, new)
                 return
@@ -1293,7 +1429,7 @@ def is_writable(path):
 
 def format_source_url(url):
     """ Format URL suitable for 'Source' stage """
-    if _HAVE_SSL:
+    if sabnzbd.HAVE_SSL:
         prot = 'https'
     else:
         prot = 'http:'
@@ -1377,31 +1513,10 @@ def set_permissions(path, recursive=True):
             set_chmod(path, umask_file, report)
 
 
-def short_path(path, always=True):
-    """ For Windows, return shortened ASCII path, for others same path
-        When `always` is off, only return a short path when size is above 259
-    """
-    if sabnzbd.WIN32:
-        import win32api
-        path = os.path.normpath(path)
-        if always or len(path) > 259:
-            # First make the path "long"
-            path = long_path(path)
-            if os.path.exists(path):
-                # Existing path can always be shortened
-                path = win32api.GetShortPathName(path)
-            else:
-                # For new path, shorten only existing part (recursive)
-                path1, name = os.path.split(path)
-                path = os.path.join(short_path(path1, always), name)
-        path = clip_path(path)
-    return path
-
-
 def clip_path(path):
-    """ Remove \\?\ or \\?\UNC\ prefix from Windows path """
+    r""" Remove \\?\ or \\?\UNC\ prefix from Windows path """
     if sabnzbd.WIN32 and path and '?' in path:
-        path = path.replace(u'\\\\?\\UNC\\', u'\\\\').replace(u'\\\\?\\', u'')
+        path = path.replace(u'\\\\?\\UNC\\', u'\\\\', 1).replace(u'\\\\?\\', u'', 1)
     return path
 
 
