@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,11 +29,12 @@ import binascii
 import shutil
 
 import sabnzbd
-from sabnzbd.encoding import TRANS, UNTRANS, unicode2local, name_fixer, \
+from sabnzbd.encoding import TRANS, UNTRANS, unicode2local, \
     reliable_unpack_names, unicoder, platform_encode, deunicode
-from sabnzbd.utils.rarfile import RarFile, is_rarfile
+import sabnzbd.utils.rarfile as rarfile
 from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv, \
-    flag_file, real_path, globber, globber_full, short_path
+    flag_file, real_path, globber, globber_full, get_all_passwords, renamer, clip_path, \
+    has_win_device
 from sabnzbd.tvsort import SeriesSorter
 import sabnzbd.cfg as cfg
 from sabnzbd.constants import Status, QCHECK_FILE, RENAMES_FILE
@@ -122,7 +123,7 @@ def find_programs(curdir):
             sabnzbd.newsunpack.PAR2_COMMAND = check(curdir, 'win/par2/par2.exe')
         if not sabnzbd.newsunpack.RAR_COMMAND:
             sabnzbd.newsunpack.RAR_COMMAND = check(curdir, 'win/unrar/UnRAR.exe')
-        sabnzbd.newsunpack.PAR2C_COMMAND = check(curdir, 'win/par2/par2-classic.exe')
+        sabnzbd.newsunpack.PAR2C_COMMAND = check(curdir, 'win/par2/par2cmdline.exe')
         sabnzbd.newsunpack.ZIP_COMMAND = check(curdir, 'win/unzip/unzip.exe')
         sabnzbd.newsunpack.SEVEN_COMMAND = check(curdir, 'win/7zip/7za.exe')
     else:
@@ -156,17 +157,13 @@ def external_processing(extern_proc, complete_dir, filename, nicename, cat, grou
                str(nicename), '', str(cat), str(group), str(status)]
 
     if failure_url:
-        command.extend(str(failure_url))
-
-    if extern_proc.endswith('.py') and (sabnzbd.WIN32 or not os.access(extern_proc, os.X_OK)):
-        command.insert(0, 'python')
-    stup, need_shell, command, creationflags = build_command(command)
-    env = fix_env()
-
-    logging.info('Running external script %s(%s, %s, %s, %s, %s, %s, %s, %s)',
-                 extern_proc, complete_dir, filename, nicename, '', cat, group, status, failure_url)
+        command.append(str(failure_url))
 
     try:
+        stup, need_shell, command, creationflags = build_command(command)
+        env = fix_env()
+        logging.info('Running external script %s(%s, %s, %s, %s, %s, %s, %s, %s)',
+                     extern_proc, complete_dir, filename, nicename, '', cat, group, status, failure_url)
         p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                             startupinfo=stup, env=env, creationflags=creationflags)
@@ -179,19 +176,24 @@ def external_processing(extern_proc, complete_dir, filename, nicename, cat, grou
     return output, ret
 
 
-def SimpleRarExtract(rarfile, name):
-    """ Extract single file from rar archive, returns (retcode, data) """
-    command = [sabnzbd.newsunpack.RAR_COMMAND, "p", "-inul", rarfile, name]
+def external_script(script, p1, p2, p3=None, p4=None):
+    """ Run a user script with two parameters, return console output and exit value """
+    command = [script, p1, p2, p3, p4]
 
-    stup, need_shell, command, creationflags = build_command(command)
-
-    p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
-                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                         startupinfo=stup, creationflags=creationflags)
+    try:
+        stup, need_shell, command, creationflags = build_command(command)
+        env = fix_env()
+        logging.info('Running user script %s(%s, %s)', script, p1, p2)
+        p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
+                            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                            startupinfo=stup, env=env, creationflags=creationflags)
+    except:
+        logging.debug("Failed script %s, Traceback: ", script, exc_info=True)
+        return "Cannot run script %s\r\n" % script, -1
 
     output = p.stdout.read()
     ret = p.wait()
-    return ret, output
+    return output, ret
 
 
 def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zips, rars, sevens, ts, depth=0):
@@ -203,7 +205,7 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zi
 
     if depth == 1:
         # First time, ignore anything in workdir_complete
-        xjoinables, xzips, xrars, xsevens, xts = build_filelists(workdir, None)
+        xjoinables, xzips, xrars, xsevens, xts = build_filelists(workdir)
     else:
         xjoinables, xzips, xrars, xsevens, xts = build_filelists(workdir, workdir_complete)
 
@@ -452,6 +454,10 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
         try:
             fail, newfiles, rars = rar_extract(rarpath, len(rar_sets[rar_set]),
                                          one_folder, nzo, rar_set, extraction_path)
+            # Was it aborted?
+            if not nzo.pp_active:
+                fail = 1
+                break
             success = not fail
         except:
             success = False
@@ -492,7 +498,7 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
     return fail, extracted_files
 
 
-def rar_extract(rarfile, numrars, one_folder, nzo, setname, extraction_path):
+def rar_extract(rarfile_path, numrars, one_folder, nzo, setname, extraction_path):
     """ Unpack single rar set 'rarfile' to 'extraction_path',
         with password tries
         Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, rars
@@ -501,39 +507,7 @@ def rar_extract(rarfile, numrars, one_folder, nzo, setname, extraction_path):
     fail = 0
     new_files = None
     rars = []
-    if nzo.password:
-        logging.info('Got a password set by user: %s', nzo.password)
-        passwords = [nzo.password.strip()]
-    else:
-        passwords = []
-        pw = nzo.nzo_info.get('password')
-        if pw:
-            passwords.append(pw)
-        # Append meta passwords, to prevent changing the original list
-        passwords.extend(nzo.meta.get('password', []))
-        if passwords:
-            logging.info('Read %s passwords from meta data in NZB', len(passwords))
-        pw_file = cfg.password_file.get_path()
-        if pw_file:
-            try:
-                pwf = open(pw_file, 'r')
-                lines = pwf.read().split('\n')
-                # Remove empty lines and space-only passwords and remove surrounding spaces
-                pws = [pw.strip('\r\n ') for pw in lines if pw.strip('\r\n ')]
-                logging.debug('Read these passwords from file: %s', pws)
-                passwords.extend(pws)
-                pwf.close()
-                logging.info('Read %s passwords from file %s', len(pws), pw_file)
-            except IOError:
-                logging.info('Failed to read the passwords file %s', pw_file)
-
-    if nzo.password:
-        # If an explicit password was set, add a retry without password, just in case.
-        passwords.append('')
-    elif not passwords or not nzo.encrypted:
-        # If we're not sure about encryption, start with empty password
-        # and make sure we have at least the empty password
-        passwords.insert(0, '')
+    passwords = get_all_passwords(nzo)
 
     for password in passwords:
         if password:
@@ -541,35 +515,27 @@ def rar_extract(rarfile, numrars, one_folder, nzo, setname, extraction_path):
             msg = T('Trying unrar with password "%s"') % unicoder(password)
             nzo.fail_msg = msg
             nzo.set_unpack_info('Unpack', msg)
-        fail, new_files, rars = rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path, password)
+        fail, new_files, rars = rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction_path, password)
         if fail != 2:
             break
 
     if fail == 2:
-        logging.error('%s (%s)', T('Unpacking failed, archive requires a password'), os.path.split(rarfile)[1])
+        logging.error('%s (%s)', T('Unpacking failed, archive requires a password'), os.path.split(rarfile_path)[1])
     return fail, new_files, rars
 
 
-def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path, password):
-    """ Unpack single rar set 'rarfile' to 'extraction_path'
+def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction_path, password):
+    """ Unpack single rar set 'rarfile_path' to 'extraction_path'
         Return fail==0(ok)/fail==1(error)/fail==2(wrong password)/fail==3(crc-error), new_files, rars
     """
     start = time()
 
     logging.debug("rar_extract(): Extractionpath: %s", extraction_path)
 
-    try:
-        zf = RarFile(rarfile)
-        expected_files = zf.unamelist()
-        zf.close()
-    except:
-        logging.info('Archive %s probably has full encryption', rarfile)
-        expected_files = []
-
     if password:
-        password = '-p%s' % password
+        password_command = '-p%s' % password
     else:
-        password = '-p-'
+        password_command = '-p-'
 
     ############################################################################
 
@@ -585,23 +551,24 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
         rename = '-or'    # Auto renaming
     if sabnzbd.WIN32:
         # Use all flags
-        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, '-ai', password,
-                   '%s' % rarfile, '%s/' % extraction_path]
+        # See: https://github.com/sabnzbd/sabnzbd/pull/771
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, '-ai', password_command,
+                   '%s' % clip_path(rarfile_path), '%s\\' % extraction_path]
     elif RAR_PROBLEM:
         # Use only oldest options (specifically no "-or")
-        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, password,
-                   '%s' % rarfile, '%s/' % extraction_path]
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, password_command,
+                   '%s' % rarfile_path, '%s/' % extraction_path]
     else:
         # Don't use "-ai" (not needed for non-Windows)
-        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, password,
-                   '%s' % rarfile, '%s/' % extraction_path]
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, password_command,
+                   '%s' % rarfile_path, '%s/' % extraction_path]
 
     if cfg.ignore_unrar_dates():
         command.insert(3, '-tsm-')
 
     stup, need_shell, command, creationflags = build_command(command)
 
-    logging.debug("Analyzing rar file ... %s found", is_rarfile(rarfile))
+    logging.debug("Analyzing rar file ... %s found", rarfile.is_rarfile(rarfile_path))
     logging.debug("Running unrar %s", command)
     p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
@@ -626,6 +593,15 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
         if not line:
             break
 
+        # Check if we should still continue
+        if not nzo.pp_active:
+            p.kill()
+            msg = T('PostProcessing was aborted (%s)') % T('Unpack')
+            nzo.fail_msg = msg
+            nzo.set_unpack_info('Unpack', msg, set=setname)
+            nzo.status = Status.FAILED
+            return fail, (), ()
+
         line = line.strip()
         lines.append(line)
 
@@ -667,6 +643,10 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
             logging.error(T('ERROR: write error (%s)'), line[11:])
             fail = 1
 
+        elif line.startswith('Cannot create') and sabnzbd.WIN32 and extraction_path.startswith('\\\\?\\'):
+            # Can be due to Unicode problems on Windows, let's retry
+            fail = 4
+
         elif line.startswith('Cannot create'):
             line2 = proc.readline()
             if 'must not exceed 260' in line2:
@@ -693,14 +673,14 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
             # unrar 4.x: "CRC failed in the encrypted file oLKQfrcNVivzdzSG22a2xo7t001.part1.rar. Corrupt file or wrong password."
             # unrar 5.x: "Checksum error in the encrypted file oLKQfrcNVivzdzSG22a2xo7t001.part1.rar. Corrupt file or wrong password."
             # unrar 5.01 : "The specified password is incorrect."
-            m = re.search('encrypted file (.+)\. Corrupt file', line)
+            m = re.search(r'encrypted file (.+)\. Corrupt file', line)
             if not m:
                 # unrar 3.x syntax
                 m = re.search(r'Encrypted file:  CRC failed in (.+) \(password', line)
             if m:
                 filename = TRANS(m.group(1)).strip()
             else:
-                filename = os.path.split(rarfile)[1]
+                filename = os.path.split(rarfile_path)[1]
             nzo.fail_msg = T('Unpacking failed, archive requires a password')
             msg = (u'[%s][%s] ' + T('Unpacking failed, archive requires a password')) % (setname, filename)
             nzo.set_unpack_info('Unpack', unicoder(msg), set=setname)
@@ -740,43 +720,19 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
             if proc:
                 proc.close()
             p.wait()
+            logging.debug('UNRAR output %s', '\n'.join(lines))
 
-            return fail, (), ()
+            # Unicode problems, lets start again but now we try without \\?\
+            # This will only fail if the download contains forbidden-Windows-names
+            if fail == 4:
+                return rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, clip_path(extraction_path), password)
+            else:
+                return fail, (), ()
 
     if proc:
         proc.close()
     p.wait()
 
-    if cfg.unpack_check():
-        if reliable_unpack_names() and not RAR_PROBLEM:
-            missing = []
-            # Loop through and check for the presence of all the files the archive contained
-            for path in expected_files:
-                if one_folder or cfg.flat_unpack():
-                    path = os.path.split(path)[1]
-                path = unicode2local(path)
-                if '?' in path:
-                    logging.info('Skipping check of file %s', path)
-                    continue
-                fullpath = os.path.join(extraction_path, path)
-                logging.debug("Checking existence of %s", fullpath)
-                if path.endswith('/'):
-                    # Folder
-                    continue
-                if not os.path.exists(fullpath):
-                    # There was a missing file, show a warning
-                    missing.append(path)
-                    logging.info(T('Missing expected file: %s => unrar error?'), path)
-
-            if missing:
-                nzo.fail_msg = T('Unpacking failed, an expected file was not unpacked')
-                logging.debug("Expecting files: %s" % str(expected_files))
-                msg = T('Unpacking failed, these file(s) are missing:') + ';' + u';'.join([unicoder(item) for item in missing])
-                nzo.set_unpack_info('Unpack', msg, set=setname)
-                return (1, (), ())
-        else:
-            logging.info('Skipping unrar file check due to unreliable file names or old unrar')
-
     logging.debug('UNRAR output %s', '\n'.join(lines))
     nzo.fail_msg = ''
     msg = T('Unpacked %s files/folders in %s') % (str(len(extracted)), format_time_string(time() - start))
@@ -856,7 +812,7 @@ def ZIP_Extract(zipfile, extraction_path, one_folder):
                '-d%s' % extraction_path]
 
     stup, need_shell, command, creationflags = build_command(command)
-
+    logging.debug('Starting unzip: %s', command)
     p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                          startupinfo=stup, creationflags=creationflags)
@@ -922,31 +878,8 @@ def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete
     """ Unpack single set 'sevenset' to 'extraction_path', with password tries
         Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, sevens
     """
-
     fail = 0
-    if nzo.password:
-        passwords = [nzo.password]
-    else:
-        passwords = []
-        pw_file = cfg.password_file.get_path()
-        if pw_file:
-            try:
-                pwf = open(pw_file, 'r')
-                passwords = pwf.read().split('\n')
-                # Remove empty lines and space-only passwords and remove surrounding spaces
-                passwords = [pw.strip('\r\n ') for pw in passwords if pw.strip('\r\n ')]
-                pwf.close()
-                logging.info('Read the passwords file %s', pw_file)
-            except IOError:
-                logging.info('Failed to read the passwords file %s', pw_file)
-
-    if nzo.password:
-        # If an explicit password was set, add a retry without password, just in case.
-        passwords.append('')
-    elif not passwords or not nzo.encrypted:
-        # If we're not sure about encryption, start with empty password
-        # and make sure we have at least the empty password
-        passwords.insert(0, '')
+    passwords = get_all_passwords(nzo)
 
     for password in passwords:
         if password:
@@ -968,7 +901,6 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
     """ Unpack single 7Z set 'sevenset' to 'extraction_path'
         Return fail==0(ok)/fail==1(error)/fail==2(wrong password), message
     """
-    msg = None
     if one_folder:
         method = 'e'  # Unpack without folders
     else:
@@ -984,17 +916,19 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
 
     if len(extensions) > 0:
         name = '%s.001' % sevenset
+        parm = '-tsplit'
     else:
         name = sevenset
+        parm = '-t7z'
 
     if not os.path.exists(name):
         return 1, T('7ZIP set "%s" is incomplete, cannot unpack') % unicoder(sevenset)
 
-    command = [SEVEN_COMMAND, method, '-y', '-aou', case, password,
+    command = [SEVEN_COMMAND, method, '-y', '-aou', parm, case, password,
                '-o%s' % extraction_path, name]
 
     stup, need_shell, command, creationflags = build_command(command)
-
+    logging.debug('Starting 7za: %s', command)
     p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                          startupinfo=stup, creationflags=creationflags)
@@ -1029,9 +963,19 @@ def par2_repair(parfile_nzf, nzo, workdir, setname, single):
     """ Try to repair a set, return readd or correctness """
     # set the current nzo status to "Repairing". Used in History
 
+    assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject)
+
+    # Check if file exists, otherwise see if another is done
+    parfile_path = os.path.join(workdir, parfile_nzf.filename)
+    if not os.path.exists(parfile_path) and parfile_nzf.extrapars:
+        for new_par in parfile_nzf.extrapars:
+            test_parfile = os.path.join(workdir, new_par.filename)
+            if os.path.exists(test_parfile):
+                parfile_nzf = new_par
+                break
+
+    # Shorten just the workdir on Windows
     parfile = os.path.join(workdir, parfile_nzf.filename)
-    parfile = short_path(parfile)
-    workdir = short_path(workdir)
 
     old_dir_content = os.listdir(workdir)
     used_joinables = ()
@@ -1054,6 +998,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname, single):
         readd = False
         for extrapar in parfile_nzf.extrapars[:]:
             parfile_nzf.extrapars.remove(extrapar)
+            parfile_nzf.nzo.remove_extrapar(extrapar)
             if extrapar not in nzo.finished_files and extrapar not in nzo.files:
                 nzo.add_parfile(extrapar)
                 readd = True
@@ -1069,7 +1014,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname, single):
             nzo.set_action_line(T('Repair'), T('Starting Repair'))
             logging.info('Scanning "%s"', parfile)
 
-            joinables, zips, rars, sevens, ts = build_filelists(workdir, None, check_rar=False)
+            joinables, zips, rars, sevens, ts = build_filelists(workdir, check_rar=False)
 
             finished, readd, pars, datafiles, used_joinables, used_par2 = PAR_Verify(parfile, parfile_nzf, nzo,
                                                                                      setname, joinables, single=single)
@@ -1137,11 +1082,13 @@ def par2_repair(parfile_nzf, nzo, workdir, setname, single):
                     logging.warning(T('Deleting %s failed!'), parfile)
 
             deletables = []
-            for f in pars:
-                if f in setpars:
-                    deletables.append(os.path.join(workdir, f))
             deletables.extend(used_joinables)
             deletables.extend(used_par2)
+
+            # Delete pars of the set and maybe extra ones that par2 found
+            deletables.extend([os.path.join(workdir, f) for f in setpars])
+            deletables.extend([os.path.join(workdir, f) for f in pars])
+
             for filepath in deletables:
                 if filepath in joinables:
                     joinables.remove(filepath)
@@ -1195,10 +1142,12 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
             command = [str(PAR2C_COMMAND), cmd, parfile]
         else:
             command = [str(PAR2_COMMAND), cmd, parfile]
+
         # Allow options if not classic or when classic and non-classic are the same
-        if options and (not classic or (PAR2_COMMAND == PAR2C_COMMAND)):
+        if (not classic or (PAR2_COMMAND == PAR2C_COMMAND)):
             command.insert(2, options)
-    logging.debug('Par2-classic = %s', classic)
+
+    logging.debug('Par2-classic/cmdline = %s', classic)
 
     # Append the wildcard for this set
     parfolder = os.path.split(parfile)[0]
@@ -1216,9 +1165,32 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
         flist = [item for item in globber_full(parfolder, wildcard) if os.path.isfile(item)]
         command.extend(flist)
 
-    stup, need_shell, command, creationflags = build_command(command)
-    logging.debug('Starting par2: %s', command)
+    # We need to check for the bad par2cmdline that skips blocks
+    # Or the one that complains about basepath
+    # Only if we're not doing multicore
+    if not tbb:
+        par2text = run_simple([command[0], '-h'])
+        if 'No data skipping' in par2text:
+            logging.info('Detected par2cmdline version that skips blocks, adding -N parameter')
+            command.insert(2, '-N')
+        if 'Set the basepath' in par2text:
+            logging.info('Detected par2cmdline version that needs basepath, adding -B<path> parameter')
+            command.insert(2, '-B')
+            command.insert(3, parfolder)
 
+    stup, need_shell, command, creationflags = build_command(command)
+
+    # par2multicore wants to see \\.\ paths on Windows
+    # But par2cmdline doesn't support that notation, or \\?\ notation
+    # See: https://github.com/sabnzbd/sabnzbd/pull/771
+    if sabnzbd.WIN32 and (tbb or has_win_device(parfile)):
+        command = [x.replace('\\\\?\\', '\\\\.\\', 1) if x.startswith('\\\\?\\') else x for x in command]
+    elif sabnzbd.WIN32:
+        # For par2cmdline on Windows we need clipped paths
+        command = [clip_path(x) if x.startswith('\\\\?\\') else x for x in command]
+
+    # Run the external command
+    logging.debug('Starting par2: %s', command)
     lines = []
     try:
         p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
@@ -1258,6 +1230,16 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
             line = linebuf.strip()
             linebuf = ''
 
+            # Check if we should still continue
+            if not nzo.pp_active:
+                p.kill()
+                msg = T('PostProcessing was aborted (%s)') % T('Repair')
+                nzo.fail_msg = msg
+                nzo.set_unpack_info('Repair', msg, set=setname)
+                nzo.status = Status.FAILED
+                readd = False
+                break
+
             # Skip empty lines
             if line == '':
                 continue
@@ -1322,7 +1304,6 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
 
                     # Move from extrapar list to files to be downloaded
                     nzo.add_parfile(nzf)
-                    extrapars.remove(nzf)
                     # Now set new par2 file as primary par2
                     nzo.partable[setname] = nzf
                     nzf.extrapars = extrapars
@@ -1471,6 +1452,15 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
                     logging.debug('PAR2 will rename "%s" to "%s"', old_name, new_name)
                     renames[new_name] = old_name
 
+                    # Show progress
+                    if verifytotal == 0 or verifynum < verifytotal:
+                        verifynum += 1
+                        nzo.set_action_line(T('Verifying'), '%02d/%02d' % (verifynum, verifytotal))
+
+            elif 'Scanning extra files' in line:
+                # Obfuscated post most likely, so reset counter to show progress
+                verifynum = 1
+
             elif 'No details available for recoverable file' in line:
                 msg = unicoder(line.strip())
                 nzo.fail_msg = msg
@@ -1531,7 +1521,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False, sin
         used_joinables.extend(reconstructed)
 
     if retry_classic:
-        logging.debug('Retry PAR2-joining with par2-classic')
+        logging.debug('Retry PAR2-joining with par2-classic/cmdline')
         return PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=True, single=single)
     else:
         return finished, readd, pars, datafiles, used_joinables, used_par2
@@ -1551,6 +1541,18 @@ def fix_env():
     else:
         return None
 
+def userxbit(filename):
+    # Returns boolean if the x-bit for user is set on the given file
+    # This is a workaround: os.access(filename, os.X_OK) does not work on certain mounted file systems
+    # Does not work on Windows, but it is not called on Windows
+
+    # rwx rwx rwx
+    # 876 543 210      # we want bit 6 from the right, counting from 0
+    userxbit = 1<<6 # bit 6
+    rwxbits = os.stat(filename)[0] # the first element of os.stat() is "mode"
+    # do logical AND, check if it is not 0:
+    xbitset = (rwxbits & userxbit) > 0
+    return xbitset
 
 def build_command(command):
     """ Prepare list from running an external program """
@@ -1559,6 +1561,16 @@ def build_command(command):
             command[n] = deunicode(command[n])
 
     if not sabnzbd.WIN32:
+        if command[0].endswith('.py'):
+            with open(command[0], 'r') as script_file:
+                if not userxbit(command[0]):
+                    # Inform user that Python scripts need x-bit and then stop
+                    logging.error(T('Python script "%s" does not have execute (+x) permission set'), command[0])
+                    raise IOError
+                elif script_file.read(2) != '#!':
+                    # No shebang (#!) defined, add default python
+                    command.insert(0, 'python')
+
         if IONICE_COMMAND and cfg.ionice().strip():
             lst = cfg.ionice().split()
             lst.reverse()
@@ -1576,6 +1588,10 @@ def build_command(command):
         creationflags = 0
 
     else:
+        # For Windows we always need to add python interpreter
+        if command[0].endswith('.py'):
+            command.insert(0, 'python')
+
         need_shell = os.path.splitext(command[0])[1].lower() not in ('.exe', '.com')
         stup = subprocess.STARTUPINFO()
         stup.dwFlags = STARTF_USESHOWWINDOW
@@ -1586,6 +1602,7 @@ def build_command(command):
         # scripts with spaces in the path don't work.
         if need_shell and ' ' in command[0]:
             command[0] = win32api.GetShortPathName(command[0])
+
         if need_shell:
             command = list2cmdline(command)
 
@@ -1622,37 +1639,60 @@ def par_sort(a, b):
         return 1
 
 
-def build_filelists(workdir, workdir_complete, check_rar=True):
+def build_filelists(workdir, workdir_complete=None, check_rar=True):
     """ Build filelists, if workdir_complete has files, ignore workdir.
         Optionally test content to establish RAR-ness
     """
-    joinables, zips, rars, sevens, filelist = ([], [], [], [], [])
+    sevens, joinables, zips, rars, ts, filelist = ([], [], [], [], [], [])
 
     if workdir_complete:
         for root, dirs, files in os.walk(workdir_complete):
             for _file in files:
                 if '.AppleDouble' not in root and '.DS_Store' not in root:
-                    filelist.append(os.path.join(root, _file))
+                    try:
+                        p = os.path.join(root, _file)
+                        filelist.append(p)
+                    except UnicodeDecodeError:
+                        # Just skip failing names
+                        pass
 
     if workdir and not filelist:
         for root, dirs, files in os.walk(workdir):
             for _file in files:
                 if '.AppleDouble' not in root and '.DS_Store' not in root:
-                    filelist.append(os.path.join(root, _file))
+                    try:
+                        p = os.path.join(root, _file)
+                        filelist.append(p)
+                    except UnicodeDecodeError:
+                        # Just skip failing names
+                        pass
 
-    sevens = [f for f in filelist if SEVENZIP_RE.search(f)]
-    sevens.extend([f for f in filelist if SEVENMULTI_RE.search(f)])
+    for file in filelist:
+        # Extra check for rar (takes CPU/disk)
+        file_is_rar = False
+        if check_rar:
+            try:
+                # Can fail on Windows due to long-path after recursive-unpack
+                file_is_rar = rarfile.is_rarfile(file)
+            except:
+                pass
 
-    if check_rar:
-        joinables = [f for f in filelist if f not in sevens and SPLITFILE_RE.search(f) and not is_rarfile(f)]
-    else:
-        joinables = [f for f in filelist if f not in sevens and SPLITFILE_RE.search(f)]
-
-    zips = [f for f in filelist if ZIP_RE.search(f)]
-
-    rars = [f for f in filelist if RAR_RE.search(f)]
-
-    ts = [f for f in filelist if TS_RE.search(f) and f not in joinables and f not in sevens]
+        # Run through all the checks
+        if SEVENZIP_RE.search(file) or SEVENMULTI_RE.search(file):
+            # 7zip
+            sevens.append(file)
+        elif SPLITFILE_RE.search(file) and not file_is_rar:
+            # Joinables, optional with RAR check
+            joinables.append(file)
+        elif ZIP_RE.search(file):
+            # ZIP files
+            zips.append(file)
+        elif RAR_RE.search(file):
+            # RAR files
+            rars.append(file)
+        elif TS_RE.search(file):
+            # TS split files
+            ts.append(file)
 
     logging.debug("build_filelists(): joinables: %s", joinables)
     logging.debug("build_filelists(): zips: %s", zips)
@@ -1672,11 +1712,14 @@ def QuickCheck(set, nzo):
 
     result = False
     nzf_list = nzo.finished_files
+    renames = {}
 
     for file in md5pack:
         found = False
+        file_platform = platform_encode(file)
         for nzf in nzf_list:
-            if file == nzf.filename:
+            # Do a simple filename based check
+            if file_platform == nzf.filename:
                 found = True
                 if (nzf.md5sum is not None) and nzf.md5sum == md5pack[file]:
                     logging.debug('Quick-check of file %s OK', file)
@@ -1685,9 +1728,27 @@ def QuickCheck(set, nzo):
                     logging.info('Quick-check of file %s failed!', file)
                     return False  # When any file fails, just stop
                 break
+
+            # Now lets do obfuscation check
+            if nzf.md5sum == md5pack[file]:
+                renames[file_platform] = nzf.filename
+                logging.debug('Quick-check renamed %s to %s', nzf.filename, file_platform)
+                renamer(os.path.join(nzo.downpath, nzf.filename), os.path.join(nzo.downpath, file_platform))
+                nzf.filename = file_platform
+                result = True
+                found = True
+                break
+
         if not found:
             logging.info('Cannot Quick-check missing file %s!', file)
             return False  # Missing file is failure
+
+    # Save renames
+    if renames:
+        previous = load_data(RENAMES_FILE, nzo.workpath, remove=False)
+        for name in previous or {}:
+            renames[name] = previous[name]
+        save_data(renames, RENAMES_FILE, nzo.workpath)
     return result
 
 
@@ -1810,12 +1871,10 @@ def pre_queue(name, pp, cat, script, priority, size, groups):
         command = [script_path, name, fix(pp), fix(cat), fix(script), fix(priority), str(size), ' '.join(groups)]
         command.extend(analyse_show(name))
 
-        stup, need_shell, command, creationflags = build_command(command)
-        env = fix_env()
-
-        logging.info('Running pre-queue script %s', command)
-
         try:
+            stup, need_shell, command, creationflags = build_command(command)
+            env = fix_env()
+            logging.info('Running pre-queue script %s', command)
             p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                                 startupinfo=stup, env=env, creationflags=creationflags)
@@ -1857,20 +1916,15 @@ def list2cmdline(lst):
     return ' '.join(nlst)
 
 
-def get_from_url(url, timeout=None):
+def get_from_url(url):
     """ Retrieve URL and return content
         `timeout` sets non-standard timeout
     """
     import urllib2
     try:
-        if timeout:
-            s = urllib2.urlopen(url, timeout=timeout)
-        else:
-            s = urllib2.urlopen(url)
-        output = s.read()
+        return urllib2.urlopen(url).read()
     except:
-        output = None
-    return output
+        return None
 
 
 def is_sevenfile(path):
@@ -1896,7 +1950,7 @@ class SevenZip(object):
                              startupinfo=stup, creationflags=creationflags)
 
         output = p.stdout.read()
-        ret = p.wait()
+        _ = p.wait()
         re_path = re.compile('^Path = (.+)')
         for line in output.split('\n'):
             m = re_path.search(line)
@@ -1923,7 +1977,7 @@ class SevenZip(object):
                              startupinfo=stup, creationflags=creationflags)
 
         output = p.stdout.read()
-        ret = p.wait()
+        _ = p.wait()
         stderr.close()
         return output
 
@@ -1934,7 +1988,7 @@ class SevenZip(object):
 
 def run_simple(cmd):
     """ Run simple external command and return output """
-    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     txt = p.stdout.read()
     p.wait()
     return txt
