@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,34 +28,31 @@ import sabnzbd
 from sabnzbd.trylist import TryList
 from sabnzbd.nzbstuff import NzbObject
 from sabnzbd.misc import exit_sab, cat_to_opts, \
-    get_admin_path, remove_all, globber, globber_full
+    get_admin_path, remove_all, globber_full
 from sabnzbd.panic import panic_queue
 import sabnzbd.database as database
 from sabnzbd.decorators import NZBQUEUE_LOCK, synchronized, synchronized_CV
 from sabnzbd.constants import QUEUE_FILE_NAME, QUEUE_VERSION, FUTURE_Q_FOLDER, JOB_ADMIN, \
     LOW_PRIORITY, NORMAL_PRIORITY, HIGH_PRIORITY, TOP_PRIORITY, \
     REPAIR_PRIORITY, STOP_PRIORITY, VERIFIED_FILE, \
-    PNFO_BYTES_FIELD, PNFO_BYTES_LEFT_FIELD, Status, QUEUE_FILE_TMPL, \
-    IGNORED_FOLDERS
+    Status, QUEUE_FILE_TMPL, \
+    IGNORED_FOLDERS, QNFO
+
 import sabnzbd.cfg as cfg
 from sabnzbd.articlecache import ArticleCache
 import sabnzbd.downloader
 from sabnzbd.assembler import Assembler, file_has_articles
-import sabnzbd.growler as growler
+import sabnzbd.notifier as notifier
 from sabnzbd.encoding import platform_encode
 from sabnzbd.bpsmeter import BPSMeter
 
 
-class NzbQueue(TryList):
+class NzbQueue:
     """ Singleton NzbQueue """
     do = None
 
     def __init__(self):
-        TryList.__init__(self)
-
-        self.__top_only = False  # cfg.top_only()
-        self.__top_nzo = None
-
+        self.__top_only = cfg.top_only()
         self.__nzo_list = []
         self.__nzo_table = {}
 
@@ -143,7 +140,7 @@ class NzbQueue(TryList):
             registered = [nzo.work_name for nzo in self.__nzo_list]
 
         # Retryable folders from History
-        items = build_history()[0]
+        items = build_history(output=True)[0]
         # Anything waiting or active or retryable is a known item
         registered.extend([platform_encode(os.path.basename(item['path']))
                            for item in items if item['retry'] or item['loaded'] or item['status'] == Status.QUEUED])
@@ -183,7 +180,7 @@ class NzbQueue(TryList):
         def all_verified(path):
             """ Return True when all sets have been successfully verified """
             verified = sabnzbd.load_data(VERIFIED_FILE, path, remove=False) or {'x': False}
-            return not bool([True for x in verified if not verified[x]])
+            return all(verified[x] for x in verified)
 
         nzo_id = None
         name = os.path.basename(folder)
@@ -230,13 +227,20 @@ class NzbQueue(TryList):
     @synchronized(NZBQUEUE_LOCK)
     def replace_in_q(self, nzo, nzo_id):
         """ Replace nzo by new in at the same spot in the queue, destroy nzo """
+        # Must be a separate function from "send_back()", due to the required queue-lock
         try:
+            old_id = nzo.nzo_id
             new_nzo = self.get_nzo(nzo_id)
             pos = self.__nzo_list.index(new_nzo)
             targetpos = self.__nzo_list.index(nzo)
             self.__nzo_list[targetpos] = new_nzo
             self.__nzo_list.pop(pos)
-            del self.__nzo_table[nzo.nzo_id]
+            # Reuse the old nzo_id
+            new_nzo.nzo_id = old_id
+            # Therefore: remove the new nzo_id
+            del self.__nzo_table[nzo_id]
+            # And attach the new nzo to the old nzo_id
+            self.__nzo_table[old_id] = new_nzo
             del nzo
             return new_nzo
         except:
@@ -252,12 +256,14 @@ class NzbQueue(TryList):
         nzo_ids = []
         # Aggregate nzo_ids and save each nzo
         for nzo in self.__nzo_list[:]:
-            if not nzo.deleted:
+            if not nzo.is_gone():
                 nzo_ids.append(os.path.join(nzo.work_name, nzo.nzo_id))
                 if save_nzo is None or nzo is save_nzo:
-                    sabnzbd.save_data(nzo, nzo.nzo_id, nzo.workpath)
                     if not nzo.futuretype:
+                        # Also includes save_data for NZO
                         nzo.save_to_disk()
+                    else:
+                        sabnzbd.save_data(nzo, nzo.nzo_id, nzo.workpath)
 
         sabnzbd.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
 
@@ -275,7 +281,7 @@ class NzbQueue(TryList):
     @synchronized(NZBQUEUE_LOCK)
     def insert_future(self, future, filename, data, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, nzo_info=None):
         """ Refresh a placeholder nzo with an actual nzo """
-        assert isinstance(future, NzbObject)
+        if 0: assert isinstance(future, NzbObject) # Assert only for debug purposes
         if nzo_info is None:
             nzo_info = {}
         nzo_id = future.nzo_id
@@ -313,8 +319,6 @@ class NzbQueue(TryList):
 
                 if cfg.auto_sort():
                     self.sort_by_avg_age()
-
-                self.reset_try_list()
             except:
                 logging.error(T('Error while adding %s, removing'), nzo_id)
                 logging.info("Traceback: ", exc_info=True)
@@ -376,7 +380,7 @@ class NzbQueue(TryList):
 
     @synchronized(NZBQUEUE_LOCK)
     def add(self, nzo, save=True, quiet=False):
-        assert isinstance(nzo, NzbObject)
+        if 0: assert isinstance(nzo, NzbObject)  # Assert only for debug purposes
         if not nzo.nzo_id:
             nzo.nzo_id = sabnzbd.get_new_id('nzo', nzo.workpath, self.__nzo_table)
 
@@ -387,7 +391,6 @@ class NzbQueue(TryList):
 
         # Reset try_lists
         nzo.reset_try_list()
-        self.reset_try_list()
 
         if nzo.nzo_id:
             nzo.deleted = False
@@ -426,7 +429,7 @@ class NzbQueue(TryList):
                 self.save(nzo)
 
             if not (quiet or nzo.status in ('Fetching',)):
-                growler.send_notification(T('NZB added to queue'), nzo.filename, 'download')
+                notifier.send_notification(T('NZB added to queue'), nzo.filename, 'download')
 
         if cfg.auto_sort():
             self.sort_by_avg_age()
@@ -443,7 +446,7 @@ class NzbQueue(TryList):
 
             if add_to_history:
                 # Create the history DB instance
-                history_db = database.get_history_handle()
+                history_db = database.HistoryDB()
                 # Add the nzo to the database. Only the path, script and time taken is passed
                 # Other information is obtained from the nzo
                 history_db.add_history_db(nzo, '', '', 0, '', '')
@@ -458,6 +461,10 @@ class NzbQueue(TryList):
                 self.save(nzo)
         else:
             nzo_id = None
+
+        # Update the last check time, since history was updated
+        sabnzbd.LAST_HISTORY_UPDATE = time.time()
+
         return nzo_id
 
     @synchronized(NZBQUEUE_LOCK)
@@ -487,7 +494,7 @@ class NzbQueue(TryList):
         return removed
 
     @synchronized(NZBQUEUE_LOCK)
-    def remove_nzf(self, nzo_id, nzf_id):
+    def remove_nzf(self, nzo_id, nzf_id, force_delete=False):
         removed = []
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
@@ -501,6 +508,13 @@ class NzbQueue(TryList):
                         self.end_job(nzo)
                     else:
                         self.remove(nzo_id, add_to_history=False, keep_basic=False)
+                elif force_delete:
+                    # Force-remove all trace
+                    nzo.bytes -= nzf.bytes
+                    nzo.bytes_tried -= (nzf.bytes - nzf.bytes_left)
+                    del nzo.files_table[nzf_id]
+                    nzo.finished_files.remove(nzf)
+
         return removed
 
     @synchronized(NZBQUEUE_LOCK)
@@ -529,6 +543,7 @@ class NzbQueue(TryList):
             handled.append(nzo_id)
         return handled
 
+    @synchronized_CV
     @synchronized(NZBQUEUE_LOCK)
     def resume_nzo(self, nzo_id):
         handled = []
@@ -538,7 +553,6 @@ class NzbQueue(TryList):
             nzo.reset_all_try_lists()
             logging.debug("Resumed nzo: %s", nzo_id)
             handled.append(nzo_id)
-        self.reset_try_list()
         return handled
 
     @synchronized(NZBQUEUE_LOCK)
@@ -737,31 +751,27 @@ class NzbQueue(TryList):
             nzf.reset_try_list()
         if nzo:
             nzo.reset_try_list()
-        self.reset_try_list()
 
     @synchronized(NZBQUEUE_LOCK)
     def reset_all_try_lists(self):
         for nzo in self.__nzo_list:
             nzo.reset_all_try_lists()
-        self.reset_try_list()
 
     @synchronized(NZBQUEUE_LOCK)
     def has_articles_for(self, server):
         """ Check whether there are any pending articles for the downloader """
         if not self.__nzo_list:
             return False
-        if self.__top_only:
-            for nzo in self.__nzo_list:
-                # Ignore any items that are in a paused or grabbing state
-                if nzo.status not in (Status.PAUSED, Status.GRABBING):
-                    return not nzo.server_in_try_list(server)
-        else:
-            # Check if this server is allowed for any object, then return if we've tried this server.
-            for nzo in self.__nzo_list:
-                if nzo.status not in (Status.PAUSED, Status.GRABBING):
-                    if nzo.server_allowed(server):
-                        return not self.server_in_try_list(server)
-            return False
+        # Check if this server is allowed for any object, then return if we've tried this server.
+        for nzo in self.__nzo_list:
+            # Not when queue paused and not a forced item
+            if (nzo.status not in (Status.PAUSED, Status.GRABBING) and not sabnzbd.downloader.Downloader.do.paused) or nzo.priority == TOP_PRIORITY:
+                # Check if past propagation delay, or forced
+                if not cfg.propagation_delay() or nzo.priority == TOP_PRIORITY or (nzo.avg_stamp + float(cfg.propagation_delay() * 60)) < time.time():
+                    # Check if category allowed
+                    if nzo.server_allowed(server) or self.__top_only:
+                        return True
+        return False
 
     @synchronized(NZBQUEUE_LOCK)
     def has_forced_items(self):
@@ -775,28 +785,20 @@ class NzbQueue(TryList):
 
     @synchronized(NZBQUEUE_LOCK)
     def get_article(self, server, servers):
-        if self.__top_only:
-            if self.__nzo_list:
-                for nzo in self.__nzo_list:
-                    # Not when queue paused and not a forced item
-                    if (nzo.status not in (Status.PAUSED, Status.GRABBING) and not sabnzbd.downloader.Downloader.do.paused) or nzo.priority == TOP_PRIORITY:
-                        if nzo.server_allowed(server):
-                            article = nzo.get_article(server, servers)
-                            if article:
-                                return article
-
-        else:
-            for nzo in self.__nzo_list:
-                # Not when queue paused and not a forced item
-                if (nzo.status not in (Status.PAUSED, Status.GRABBING) and not sabnzbd.downloader.Downloader.do.paused) or nzo.priority == TOP_PRIORITY:
-                    # Don't try to get an article if server is in try_list of nzo
-                    if not nzo.server_in_try_list(server) and nzo.server_allowed(server):
+        for nzo in self.__nzo_list:
+            # Not when queue paused and not a forced item
+            if (nzo.status not in (Status.PAUSED, Status.GRABBING) and not sabnzbd.downloader.Downloader.do.paused) or nzo.priority == TOP_PRIORITY:
+                # Check if past propagation delay, or forced
+                if not cfg.propagation_delay() or nzo.priority == TOP_PRIORITY or (nzo.avg_stamp + float(cfg.propagation_delay() * 60)) < time.time():
+                    # Don't try to get an article if server is in try_list of nzo and category allowed by server
+                    if nzo.server_allowed(server) and not nzo.server_in_try_list(server):
                         article = nzo.get_article(server, servers)
                         if article:
                             return article
-
-            # No articles for this server, block server (until reset issued)
-            self.add_to_try_list(server)
+                    # Stop after first job that wasn't paused/propagating/etc
+                    # But make sure not to get stuck behind bad category
+                    if self.__top_only and not nzo.server_allowed(server):
+                        return
 
     @synchronized(NZBQUEUE_LOCK)
     def register_article(self, article, found=True):
@@ -811,10 +813,7 @@ class NzbQueue(TryList):
 
         filename = nzf.filename
 
-        if reset:
-            self.reset_try_list()
-
-        if nzo.status in (Status.COMPLETED, Status.DELETED):
+        if nzo.is_gone():
             logging.debug('Discarding file completion %s for deleted job', filename)
         else:
             if file_done:
@@ -877,29 +876,37 @@ class NzbQueue(TryList):
         return n
 
     @synchronized(NZBQUEUE_LOCK)
-    def queue_info(self, for_cli=False, max_jobs=0, search=None):
-        """ Return list of queued jobs, optionally filtered by 'search' """
+    def queue_info(self, search=None, start=0, limit=0):
+        """ Return list of queued jobs,
+            optionally filtered by 'search' and limited by start and limit.
+        """
         if search:
             search = search.lower()
         bytes_left = 0
         bytes_total = 0
+        bytes_left_previous_page = 0
         q_size = 0
         pnfo_list = []
         n = 0
+
         for nzo in self.__nzo_list:
-            if nzo.status != 'Paused':
-                b, b_left = nzo.total_and_remaining()
-                bytes_total += b
+            if nzo.status not in (Status.PAUSED, Status.CHECKING):
+                b_left = nzo.remaining()
+                bytes_total += nzo.bytes
                 bytes_left += b_left
                 q_size += 1
+                # We need the number of bytes before the current page
+                if n < start:
+                    bytes_left_previous_page += b_left
 
             if (not search) or search in nzo.final_name_pw_clean.lower():
-                if not max_jobs or n < max_jobs:
-                    pnfo = nzo.gather_info(for_cli=for_cli)
-                    pnfo_list.append(pnfo)
+                if (not limit) or (start <= n < start+limit):
+                    pnfo_list.append(nzo.gather_info())
                 n += 1
 
-        return (bytes_total, bytes_left, pnfo_list, q_size, len(self.__nzo_list))
+        if not search:
+            n = len(self.__nzo_list)
+        return QNFO(bytes_total, bytes_left, bytes_left_previous_page, pnfo_list, q_size, n)
 
     @synchronized(NZBQUEUE_LOCK)
     def remaining(self):
