@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-
 """
 sabtray.py - Systray icon for SABnzbd on Windows, contributed by Jan Schejbal
 """
@@ -29,22 +28,21 @@ import sabnzbd.api as api
 import sabnzbd.scheduler as scheduler
 from sabnzbd.downloader import Downloader
 import sabnzbd.cfg as cfg
-from sabnzbd.constants import MEBI
 from sabnzbd.misc import to_units
 import os
 import cherrypy
 
+# contains the tray icon, which demands its own thread
 from sabnzbd.utils.systrayiconthread import SysTrayIconThread
 
-# contains the tray icon, which demands its own thread
+
 class SABTrayThread(SysTrayIconThread):
     sabicons = {
-        'default': 'icons/sabnzbd16.ico',
-        'green': 'icons/sabnzbd16green.ico',
-        'pause': 'icons/sabnzbd16paused.ico'
+        'default': 'icons/sabnzbd16_32.ico',
+        'green': 'icons/sabnzbd16_32green.ico',
+        'pause': 'icons/sabnzbd16_32paused.ico'
     }
 
-
     def __init__(self):
         # Wait for translated texts to be loaded
         while not sabnzbd.WEBUI_READY:
@@ -55,18 +53,40 @@ class SABTrayThread(SysTrayIconThread):
         self.counter = 0
         text = "SABnzbd"
 
+        self.set_texts()
         menu_options = (
-            (T('Show interface'), None, self.browse),
-            (T('Open complete folder'), None, self.opencomplete),
-            (T('Troubleshoot'), None, ((T('Restart'), None, self.restart),
-                                      (T('Restart without login'), None, self.nologin),
-                                      (T('Restart') + ' - 127.0.0.1:8080', None, self.defhost))),
-            (T('Pause') + '/' + T('Resume'), None, self.pauseresume),
-            (T('Shutdown'), None, self.shutdown),
+            (self.txt_show_int, None, self.browse),
+            (self.txt_open_comp, None, self.opencomplete),
+            (self.txt_trouble, None, ((self.txt_restart, None, self.restart),
+                                      (self.txt_restart_nl, None, self.nologin),
+                                      (self.txt_restart + ' - 127.0.0.1:8080', None, self.defhost))),
+            (self.txt_pause + '/' + self.txt_resume, None, self.pauseresume),
+            (self.txt_rss, None, self.rss),
+            (self.txt_shutdown, None, self.shutdown),
         )
 
         SysTrayIconThread.__init__(self, self.sabicons['default'], text, menu_options, None, 0, "SabTrayIcon")
 
+    def set_texts(self):
+        def fix(txt):
+            if trans:
+                return Tx(txt)
+            else:
+                return txt
+
+        trans = str(get_codepage()) == str(sabnzbd.lang.CODEPAGE)
+        self.txt_show_int = fix(TT('Show interface'))
+        self.txt_open_comp = fix(TT('Open complete folder'))
+        self.txt_trouble = fix(TT('Troubleshoot'))
+        self.txt_pause = fix(TT('Pause'))
+        self.txt_shutdown = fix(TT('Shutdown'))
+        self.txt_resume = fix(TT('Resume'))
+        self.txt_restart = fix(TT('Restart'))
+        self.txt_restart_nl = fix(TT('Restart without login'))
+        self.txt_idle = fix(TT('Idle'))
+        self.txt_paused = fix(TT('Paused'))
+        self.txt_remaining = fix(TT('Remaining'))
+        self.txt_rss = fix(TT('Read all RSS feeds'))
 
     # called every few ms by SysTrayIconThread
     def doUpdates(self):
@@ -78,19 +98,17 @@ class SABTrayThread(SysTrayIconThread):
             speed = to_units(bpsnow, dec_limit=1)
 
             if self.sabpaused:
-                self.hover_text = T('Paused')
+                self.hover_text = self.txt_paused
                 self.icon = self.sabicons['pause']
             elif bytes_left > 0:
-                self.hover_text = "%sB/s %s: %sB (%s)" % (speed, T('Remaining'), mb_left, time_left)
+                self.hover_text = "%sB/s %s: %sB (%s)" % (speed, self.txt_remaining, mb_left, time_left)
                 self.icon = self.sabicons['green']
             else:
-                self.hover_text = T('Idle')
+                self.hover_text = self.txt_idle
                 self.icon = self.sabicons['default']
 
             self.refresh_icon()
             self.counter = 0
-        if sabnzbd.SABSTOP:
-            self.terminate = True
 
     # menu handler
     def opencomplete(self, icon):
@@ -112,31 +130,33 @@ class SABTrayThread(SysTrayIconThread):
 
     # menu handler
     def restart(self, icon):
-        self.hover_text = T('Restart')
-        sabnzbd.halt()
-        cherrypy.engine.restart()
+        self.hover_text = self.txt_restart
+        sabnzbd.trigger_restart()
+
+    # menu handler
+    def rss(self, icon):
+        self.hover_text = self.txt_rss
+        scheduler.force_rss()
 
     # menu handler
     def nologin(self, icon):
         sabnzbd.cfg.username.set('')
         sabnzbd.cfg.password.set('')
         sabnzbd.config.save_config()
-        self.hover_text = T('Restart')
-        sabnzbd.halt()
-        cherrypy.engine.restart()
+        self.hover_text = self.txt_restart
+        sabnzbd.trigger_restart()
 
     # menu handler
     def defhost(self, icon):
         sabnzbd.cfg.cherryhost.set('127.0.0.1')
         sabnzbd.cfg.enable_https.set(False)
         sabnzbd.config.save_config()
-        self.hover_text = T('Restart')
-        sabnzbd.halt()
-        cherrypy.engine.restart()
+        self.hover_text = self.txt_restart
+        sabnzbd.trigger_restart()
 
     # menu handler - adapted from interface.py
     def shutdown(self, icon):
-        self.hover_text = T('Shutdown')
+        self.hover_text = self.txt_shutdown
         sabnzbd.halt()
         cherrypy.engine.exit()
         sabnzbd.SABSTOP = True
@@ -151,3 +171,9 @@ class SABTrayThread(SysTrayIconThread):
         scheduler.plan_resume(0)
         sabnzbd.unpause_all()
 
+
+def get_codepage():
+    import locale
+    _lang, code = locale.getlocale()
+    logging.debug('SysTray uses codepage %s', code)
+    return code
