@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,11 +21,11 @@ sabnzbd.utils.servertests - Debugging server connections. Currently only NNTP se
 
 import socket
 import sys
+import select
 
 from sabnzbd.newswrapper import NewsWrapper
 from sabnzbd.downloader import Server, clues_login, clues_too_many
 from sabnzbd.config import get_servers
-from sabnzbd.encoding import xml_name
 from sabnzbd.misc import int_conv
 
 
@@ -41,7 +41,7 @@ def test_nntp_server_dict(kwargs):
     if not connections:
         return False, T('There are no connections set. Please set at least one connection.')
     ssl = int_conv(kwargs.get('ssl', 0))
-    ssl_type = kwargs.get('ssl_type', 't1')
+    ssl_verify = int_conv(kwargs.get('ssl_verify', 1))
     port = int_conv(kwargs.get('port', 0))
     if not port:
         if ssl:
@@ -50,10 +50,10 @@ def test_nntp_server_dict(kwargs):
             port = 119
 
     return test_nntp_server(host, port, server, username=username,
-                        password=password, ssl=ssl, ssl_type=ssl_type)
+                        password=password, ssl=ssl, ssl_verify=ssl_verify)
 
 
-def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None, ssl_type='t1'):
+def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None, ssl_verify=1):
     """ Will connect (blocking) to the nttp server and report back any errors """
     timeout = 4.0
     if '*' in password and not password.strip('*'):
@@ -74,7 +74,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
         if not got_pass:
             return False, T('Password masked in ******, please re-enter')
     try:
-        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_type, False, username, password)
+        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_verify, False, username, password)
     except:
         return False, T('Invalid server details')
 
@@ -84,6 +84,10 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
         while not nw.connected:
             nw.lines = []
             nw.recv_chunk(block=True)
+            #more ssl related: handle 1/n-1 splitting to prevent Rizzo/Duong-Beast
+            read_sockets, _, _ = select.select([nw.nntp.sock], [], [], 0.1)
+            if read_sockets:
+                nw.recv_chunk(block=True)
             nw.finish_connect(nw.lines[0][:3])
 
     except socket.timeout, e:
@@ -91,7 +95,12 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
             return False, T('Timed out: Try enabling SSL or connecting on a different port.')
         else:
             return False, T('Timed out')
+
     except socket.error, e:
+        # Trying SSL on non-SSL port?
+        if 'unknown protocol' in str(e).lower():
+            return False, T('Unknown SSL protocol: Try disabling SSL or connecting on a different port.')
+
         return False, unicode(e)
 
     except TypeError, e:
