@@ -1,8 +1,8 @@
 # This file will be patched by setup.py
 # The __version__ should be set to the branch name
-# (e.g. "trunk" or "0.4.x")
+# (e.g. "develop" or "1.2.x")
 
 # You MUST use double quotes (so " and not ')
 
-__version__ = "0.7.11"
-__baseline__ = "a50412a32751f4d1d555020dea20a2da43d6a309"
+__version__ = "1.2.1"
+__baseline__ = "d32cf57c7573ed43bcab40eb9b23a0853f1f7015"
